#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
import re
from configparser import ConfigParser, NoSectionError
from time import sleep

import dbus

from setting.globalconfig import GlobalConfig
from src import logger
from src.cmdctl import CmdCtl
from src.custom_exception import ApplicationStartError
from src.custom_exception import GetWindowInformation
from src.custom_exception import NoSetReferencePoint
from src.shortcut import ShortCut
from src.wayland_wininfo import WaylandWindowINfo


class ButtonCenter:
    """
    根据应用程序中控件元素的相对坐标，通过配置元素的x、y、w和h来定位元素在屏幕中的位置，并返回用于鼠标和键盘操作的坐标。
    """
    # pylint: disable=too-many-arguments,too-many-locals,too-many-public-methods
    __author__ = "Mikigo <huangmingqiang@uniontech.com>, Litao <litaoa@uniontech.com>"

    def __init__(self, app_name: str, config_path: str, number: int = -1, pause: int = 1):
        """
        :param app_name: 系统应用软件包，例如，dde-file-manager
        :param config_path: ui 定位配置文件路径（绝对路径）
        :param number: 默认为 -1, 即最后一个窗口
            如果你想指定不同的窗口，你可以在实例化对象的时候显式的传入 number，第一个为 0
        """
        self.app_name = app_name
        self.number = number
        # 每个操作步骤之前暂停的时间
        self.pause = pause
        self.config_path = config_path

    def window_info(self):
        """
         窗口信息
        :return:  窗口的基本信息，左上角坐标，窗口宽高等
        """
        if GlobalConfig.IS_X11:
            try:
                # Get window_operate ID based on package name
                app_id = CmdCtl.run_cmd(
                    f"xdotool search --classname --onlyvisible {self.app_name}",
                    interrupt=False,
                    out_debug_flag=False,
                    command_log=False,
                ).split("\n")
                app_id_list = [int(_id) for _id in app_id if _id]
                app_id_list.sort()
                logger.debug(f"app_id_list: {app_id_list}")
                return CmdCtl.run_cmd(
                    f"xwininfo -id {app_id_list[self.number]}",
                    interrupt=False,
                    out_debug_flag=False,
                    command_log=False,
                )
            except Exception as exc:
                raise ApplicationStartError(f"{self.app_name, exc}") from exc

        elif GlobalConfig.IS_WAYLAND:
            # 移动到当前窗口
            proxy_object = dbus.SessionBus().get_object("org.kde.KWin", "/dde")
            # 移动鼠标到目标窗口
            dbus.Interface(proxy_object, "org.kde.KWin").WindowMove()
            sleep(self.pause)
            ShortCut.esc()
            return WaylandWindowINfo().window_info()
        return None

    def window_location_and_sizes(self):
        """
         获取窗口的位置及大小
        :return:
        """
        try:
            app_window_info = self.window_info()

            if GlobalConfig.IS_X11:
                re_pattern = re.compile(r"Absolute.*:\s\s(-?\d+)")
                result = re.findall(re_pattern, app_window_info)
                window_width = re.findall(r"Width.*:\s(\d+)", app_window_info)[0]
                window_height = re.findall(r"Height.*:\s(\d+)", app_window_info)[0]
                if not result:
                    sleep(1)
                    app_window_info = self.window_info()
                    result = re.findall(re_pattern, app_window_info)
                    window_width = re.findall(r"Width.*:\s(\d+)", app_window_info)[0]
                    window_height = re.findall(r"Height.*:\s(\d+)", app_window_info)[0]
                window_x, window_y = result
            # wayland
            else:
                name = app_window_info.get("name")
                if name != self.app_name:
                    raise ValueError(f"您想要获取的窗口为：{self.app_name}, 但实际获取的窗口为：{name}")
                window_x, window_y, window_width, window_height = app_window_info.get("wininfo")
            logger.debug(
                f"窗口左上角坐标 {window_x, window_y},获取窗口大小 {window_width}*{window_height}"
            )
            return (int(window_x), int(window_y), int(window_width), int(window_height))
        except (IndexError, KeyError) as exc:
            raise GetWindowInformation(f"获取窗口大小错误 {exc}") from exc

    def window_left_top_position(self) -> tuple:
        """
         获取窗口左上角坐标
        :return:  (0, 0)
        """
        try:
            re_pattern = re.compile(r"Absolute.*:\s\s(-?\d+)")
            app_window_info = self.window_info()
            if GlobalConfig.IS_X11:
                result = re.findall(re_pattern, app_window_info)
                if not result:
                    sleep(1)
                    result = re.findall(re_pattern, self.window_info())
                window_x, window_y = result
            else:
                window_x, window_y, window_width, window_height = app_window_info.get("wininfo")
            logger.debug(f"窗口左上角坐标 {window_x, window_y}")
            return int(window_x), int(window_y)
        except (ValueError, KeyError) as exc:
            raise GetWindowInformation(f"获取窗口左上角坐标错误 {exc}") from exc

    def window_sizes(self) -> tuple:
        """
         获取窗口的大小
        :return:  (宽， 高)， 例如：(400, 600)
        """
        try:
            app_window_info = self.window_info()
            if GlobalConfig.IS_X11:
                window_width = re.findall(r"Width.*:\s(\d+)", app_window_info)[0]
                window_height = re.findall(r"Height.*:\s(\d+)", app_window_info)[0]
            else:
                window_x, window_y, window_width, window_height = app_window_info.get("wininfo")
            logger.debug(f"获取窗口大小 {window_width}*{window_height}")
            return int(window_width), int(window_height)
        except (IndexError, KeyError) as exc:
            raise GetWindowInformation(f"获取窗口大小错误 {exc}") from exc

    def window_left_bottom_position(self) -> tuple:
        """
         左下角的坐标
        :return:  (0, 1080)
        """
        (
            window_x,
            window_y,
            _window_width,
            window_height,
        ) = self.window_location_and_sizes()
        left_y = window_y + window_height
        logger.debug(f"窗口左下角坐标 {window_x, left_y}")
        return int(window_x), int(left_y)

    def window_right_top_position(self) -> tuple:
        """
         右上角的坐标
        :return:  (1920, 0)
        """
        (
            window_x,
            window_y,
            window_width,
            _window_height,
        ) = self.window_location_and_sizes()
        right_x = window_x + window_width
        logger.debug(f"窗口右上角坐标 {right_x, window_y}")
        return int(right_x), int(window_y)

    def window_right_bottom_position(self) -> tuple:
        """
         右下角的坐标
        :return:  (1920, 1080)
        """
        (
            window_x,
            window_y,
            window_width,
            window_height,
        ) = self.window_location_and_sizes()
        right_x = window_x + window_width
        right_y = window_y + window_height
        logger.debug(f"窗口右下角坐标 {right_x, right_y}")
        return int(right_x), int(right_y)

    def window_left_center_position(self) -> tuple:
        """
         获取窗口左边界中心坐标
        :return:  (0, 540)
        """
        (
            window_x,
            window_y,
            _window_width,
            window_height,
        ) = self.window_location_and_sizes()
        center_y = window_y + window_height / 2
        logger.debug(f"窗口左边界中心坐标 {window_x, center_y}")
        return int(window_x), int(center_y)

    def window_top_center_position(self) -> tuple:
        """
         获取窗口上边界中心坐标
        :return:  (960, 0)
        """
        (
            window_x,
            window_y,
            window_width,
            _window_height,
        ) = self.window_location_and_sizes()
        center_x = window_x + window_width / 2
        logger.debug(f"获取窗口上边界中心坐标 {center_x, window_y}")
        return int(center_x), int(window_y)

    def window_right_center_position(self) -> tuple:
        """
         获取窗口右边界中心坐标
        :return:  (1920, 540)
        """
        (
            window_x,
            window_y,
            window_width,
            window_height,
        ) = self.window_location_and_sizes()
        right_x = window_x + window_width
        center_y = window_y + window_height / 2
        logger.debug(f"获取窗口右边界中心坐标 {right_x, center_y}")
        return int(right_x), int(center_y)

    def window_bottom_center_position(self) -> tuple:
        """
         获取窗口下边界中心的坐标
        :return:  (960, 1080)
        """
        (
            window_x,
            window_y,
            window_width,
            window_height,
        ) = self.window_location_and_sizes()
        center_x = window_x + window_width / 2
        bottom_y = window_y + window_height
        logger.debug(f"获取窗口下边界中心的坐标 {center_x, bottom_y}")
        return int(center_x), int(bottom_y)

    def window_center(self) -> tuple:
        """
         获取窗口的中心点坐标
        :return:  (960, 540)
        """
        (
            window_x,
            window_y,
            window_width,
            window_height,
        ) = self.window_location_and_sizes()
        _x = window_x + window_width / 2
        _y = window_y + window_height / 2
        logger.debug(f"窗口中心坐标 {_x, _y}")
        return _x, _y

    def btn_center_by_left_top(self, button_x, button_y, button_w, button_h) -> tuple:
        """
         根据左上角的坐标按钮的中心坐标
        :param button_x: 控件左上角相对于窗口左上角的横向距离
        :param button_y: 控件左上角相对于窗口左上角的纵向距离
        :param button_w: 控件宽度
        :param button_h: 控件高度
        :return:  控件的中心坐标 （1, 1）
        """
        window_x, window_y = self.window_left_top_position()
        b_x = window_x + button_x + button_w / 2
        b_y = window_y + button_y + button_h / 2
        logger.debug(f"左上角按钮的中心坐标 {b_x, b_y}")
        return b_x, b_y

    def btn_center_by_right_top(self, button_x, button_y, button_w, button_h) -> tuple:
        """
         根据右上角的坐标按钮的中心坐标
        :param button_x: 控件右上角相对于窗口右上角的横向距离
        :param button_y: 控件右上角相对于窗口右上角的纵向距离
        :param button_w: 控件宽度
        :param button_h: 控件高度
        :return:  控件的中心坐标 （1, 1）
        """
        window_x, window_y = self.window_right_top_position()
        b_x = window_x - button_x - button_w / 2
        b_y = window_y + button_y + button_h / 2
        logger.debug(f"右上角按钮的中心坐标 {b_x, b_y}")
        return b_x, b_y

    def btn_center_by_left_bottom(
            self, button_x, button_y, button_w, button_h
    ) -> tuple:
        """
         根据左下角的坐标按钮的中心坐标
        :param button_x: 控件左下角相对于窗口左下角的横向距离
        :param button_y: 控件左下角相对于窗口左下角的纵向距离
        :param button_w: 控件宽度
        :param button_h: 控件高度
        :return:  控件的中心坐标 （1, 1）
        """
        window_x, window_y = self.window_left_bottom_position()
        b_x = window_x + button_x + button_w / 2
        b_y = window_y - button_y - button_h / 2
        logger.debug(f"左下角按钮的中心坐标 {b_x, b_y}")
        return b_x, b_y

    def btn_center_by_right_bottom(
            self, button_x, button_y, button_w, button_h
    ) -> tuple:
        """
         根据右下角的坐标按钮的中心坐标
        :param button_x: 控件右下角相对于窗口右下角的横向距离
        :param button_y: 控件右下角相对于窗口右下角的纵向距离
        :param button_w: 控件宽度
        :param button_h: 控件高度
        :return:  控件的中心坐标 （1, 1）
        """
        window_x, window_y = self.window_right_bottom_position()
        b_x = window_x - button_x - button_w / 2
        b_y = window_y - button_y - button_h / 2
        logger.debug(f"右下角按钮的中心坐标 {b_x, b_y}")
        return b_x, b_y

    def btn_pic_by_left_top(self, button_x, button_y, button_w, button_h) -> tuple:
        """
         根据左上角的坐标按钮的截图区域
        :param button_x: 控件左上角相对于窗口左上角的横向距离
        :param button_y: 控件左上角相对于窗口左上角的纵向距离
        :param button_w: 控件宽度
        :param button_h: 控件高度
        :return:  控件的中心坐标 （1, 1）
        """
        window_x, window_y = self.window_left_top_position()
        b_x = window_x + button_x
        b_y = window_y + button_y
        logger.debug(f"左上角按钮的截取区域左上角 {b_x, b_y}, 控件长宽 {button_w, button_h}")
        return b_x, b_y, button_w, button_h

    def btn_pic_by_right_top(self, button_x, button_y, button_w, button_h) -> tuple:
        """
         根据右上角的坐标按钮的截图区域
        :param button_x: 控件右上角相对于窗口右上角的横向距离
        :param button_y: 控件右上角相对于窗口右上角的纵向距离
        :param button_w: 控件宽度
        :param button_h: 控件高度
        :return:  控件的中心坐标 （1, 1）
        """
        window_x, window_y = self.window_right_top_position()
        b_x = window_x - button_x - button_w
        b_y = window_y + button_y
        logger.debug(f"右上角按钮的截取区域左上角 {b_x, b_y}, 控件长宽 {button_w, button_h}")
        return b_x, b_y, button_w, button_h

    def btn_pic_by_left_bottom(self, button_x, button_y, button_w, button_h) -> tuple:
        """
         根据左下角的坐标按钮的截图区域
        :param button_x: 控件左下角相对于窗口左下角的横向距离
        :param button_y: 控件左下角相对于窗口左下角的纵向距离
        :param button_w: 控件宽度
        :param button_h: 控件高度
        :return:  控件的中心坐标 （1, 1）
        """
        window_x, window_y = self.window_left_bottom_position()
        b_x = window_x + button_x
        b_y = window_y - button_y - button_h
        logger.debug(f"左下角按钮的截取区域左上角 {b_x, b_y}, 控件长宽 {button_w, button_h}")
        return b_x, b_y, button_w, button_h

    def btn_pic_by_right_bottom(self, button_x, button_y, button_w, button_h) -> tuple:
        """
         根据右下角的坐标按钮的截图区域
        :param button_x: 控件右下角相对于窗口右下角的横向距离
        :param button_y: 控件右下角相对于窗口右下角的纵向距离
        :param button_w: 控件宽度
        :param button_h: 控件高度
        :return:  控件的中心坐标 （1, 1）
        """
        window_x, window_y = self.window_right_bottom_position()
        b_x = window_x - button_x - button_w
        b_y = window_y - button_y - button_h
        logger.debug(f"右下角按钮的截取区域左上角 {b_x, b_y}, 控件长宽 {button_w, button_h}")
        return b_x, b_y, button_w, button_h

    def btn_center(
            self,
            btn_name,
            offset_x=None,
            multiplier_x=None,
            offset_y=None,
            multiplier_y=None,
    ) -> tuple:
        """
         获取元素的中心坐标
        :param btn_name: 控件名
        :param offset_x
            正数为右移动
            负数为左移动
        :param multiplier_x
            offset_x 移动的倍数
        :param offset_y
            正数为上移动
            负数为下移动
        :param multiplier_y
            offset_y 移动的倍数
        """
        btn_x = btn_y = ""
        sleep(self.pause)
        conf = ConfigParser()
        if isinstance(self.config_path, list):
            for config in self.config_path:
                conf.read(config)
        elif isinstance(self.config_path, str):
            conf.read(self.config_path)
        else:
            raise ValueError
        try:
            direction = conf.get(btn_name, "direction")
        except NoSectionError:
            raise NoSectionError(f"在 [{self.config_path}] 文件中没有配置 '{btn_name}'")
        position = [int(i.strip()) for i in conf.get(btn_name, "location").split(",")]
        default_point = ("left_bottom", "left_top", "right_top", "right_bottom")
        default_boundary_point = (
            "top_center",
            "bottom_center",
            "left_center",
            "right_center",
        )
        if direction in default_point:
            btn_x, btn_y = getattr(self, f"btn_center_by_{direction}")(*position)
        elif direction in default_boundary_point:
            window_x, window_y = getattr(self, f"window_{direction}_position")()
            # pylint: disable=eval-used
            btn_x = eval(
                f"{window_x} + {position[0]} {'+' if position[0] > 0 else '-'} {position[2] / 2}"
            )
            # pylint: disable=eval-used
            btn_y = eval(
                f"{window_y} + {position[1]} {'+' if position[1] > 0 else '-'} {position[3] / 2}"
            )
        elif direction == "window_size":
            btn_x, btn_y, button_w, button_y = self.window_location_and_sizes()
            btn_x = btn_x + button_w / 2
            btn_y = btn_y + button_y / 2
        if btn_x and btn_y:
            if offset_x:
                btn_x = btn_x + int(offset_x) * (
                    int(multiplier_x) if multiplier_x else 1
                )
            if offset_y:
                btn_y = btn_y + int(offset_y) * (
                    int(multiplier_y) if multiplier_y else 1
                )
            logger.debug(f"[{btn_name}] 坐标：{str(btn_x)}, {str(btn_y)})")
            return btn_x, btn_y
        raise NoSetReferencePoint(
            f"{direction}, 默认参考点 {default_point + default_boundary_point}"
        )

    def btn_size(
            self,
            btn_name: str,
            offset_x: [int, float] = None,
            multiplier_x: [int, float] = None,
            offset_y: [int, float] = None,
            multiplier_y: [int, float] = None,
    ) -> tuple:
        """
         获取元素的左上角坐标及长宽
        :param btn_name: 控件名
        :param offset_x
            正数为右移动
            负数为左移动
        :param multiplier_x
            offset_x 移动的倍数
        :param offset_y
            正数为上移动
            负数为下移动
        :param multiplier_y
            offset_y 移动的倍数
        """
        btn_x = btn_y = button_w = button_y = ""
        sleep(self.pause)
        conf = ConfigParser()
        conf.read(self.config_path)
        direction = conf.get(btn_name, "direction")
        position = [int(i.strip()) for i in conf.get(btn_name, "location").split(",")]
        default_point = ("left_bottom", "left_top", "right_top", "right_bottom")
        default_boundary_point = (
            "top_center",
            "bottom_center",
            "left_center",
            "right_center",
        )
        if direction in default_point:
            btn_x, btn_y, button_w, button_y = getattr(self, f"btn_pic_by_{direction}")(
                *position
            )
        elif direction in default_boundary_point:
            window_x, window_y = getattr(self, f"window_{direction}_position")()
            btn_x = window_x + position[0] - (0 if position[0] > 0 else position[2])
            btn_y = window_y + position[1] - (0 if position[1] > 0 else position[3])
            button_w, button_y = position[2], position[3]
        elif direction == "window_size":
            btn_x, btn_y, button_w, button_y = self.window_location_and_sizes()
        if btn_x != "" and btn_y != "":
            if offset_x:
                btn_x = btn_x + int(offset_x) * (
                    int(multiplier_x) if multiplier_x else 1
                )
            if offset_y:
                btn_y = btn_y + int(offset_y) * (
                    int(multiplier_y) if multiplier_y else 1
                )
            logger.debug(
                f"[{btn_name}] 左上角坐标：{str(btn_x)}, {str(btn_y)}), 长宽 {button_w, button_y}"
            )
            return btn_x, btn_y, button_w, button_y
        raise NoSetReferencePoint(
            f"{direction}, 默认参考点 {default_point + default_boundary_point}"
        )

    def btn_info(self, btn_name: str) -> tuple:
        """
         元素的相对位置和参考系
        :param btn_name: 控件名称
        :return: (相对坐标，参考系）
        """
        conf = ConfigParser()
        conf.read(self.config_path)
        direction = conf.get(btn_name, "direction")
        position = [int(i.strip()) for i in conf.get(btn_name, "location").split(",")]
        return position, direction

    @classmethod
    def get_windows_number(cls, name: str) -> int:
        """
         获取应用所有窗口数量
        :param name: 应用包名
        :return: int 窗口数量
        """
        cmd = f"xdotool search --classname {name}"
        app_id = CmdCtl.run_cmd(
            cmd, interrupt=False, out_debug_flag=False, command_log=False
        )
        return len([i for i in app_id.split("\n") if i])

    @classmethod
    def get_windows_id(cls, name: str) -> list:
        """
         获取活动应用窗口ID
        :param name: 应用包名
        :return: 窗口编号列表
        """
        cmd = f"xdotool search --onlyvisible --classname {name}"
        app_id = CmdCtl.run_cmd(
            cmd, interrupt=False, out_debug_flag=False, command_log=False
        )
        if app_id:
            return [i for i in app_id.split("\n") if i]
        raise ApplicationStartError(app_id)

    def focus_windows(self, app_name: str = None):
        """
         窗口置顶并聚焦
        :param app_name: 应用包名
        """
        if GlobalConfig.IS_WAYLAND:
            return
        app_id = self.get_windows_id(app_name if app_name else self.app_name)
        windows = int(app_id[self.number])
        cmd = f"xdotool windowactivate {windows}"
        CmdCtl.run_cmd(cmd, interrupt=False, out_debug_flag=False, command_log=False)
        logger.debug(f"<{app_name}> 窗口置顶并聚焦")

    @staticmethod
    def get_lastest_window_id(app_name: str) -> int:
        """
         获取应用的所有窗口编号，并返回编号最大的窗口ID
        :return: 返回最新创建的窗口编号
        """
        try:
            app_id = CmdCtl.run_cmd(
                f"xdotool search --classname --onlyvisible {app_name}",
                interrupt=False,
                out_debug_flag=False,
                command_log=False,
            ).split("\n")
            app_id_list = [int(_id) for _id in app_id if _id]  # to int
            app_id_list.sort()
            return app_id_list[-1]
        except Exception as exc:
            raise ApplicationStartError(f"{app_name, exc}") from exc



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=C0301,C0116,R1710,W0201,R0903
import json
import re
from collections import deque

from src  import logger
from src.pms._base import _Base
from src.pms._base import _unicode_to_cn
from src.pms._base import MAX_CASE_NUMBER


class Suite(_Base):
    """获取测试套件关联的用例"""

    __author__ = "huangmingqiang@uniontech.com"

    def get_suite_data(self, suite_id):
        url = f"https://pms.uniontech.com/testsuite-view-{suite_id}-id_desc-1-{MAX_CASE_NUMBER}-1.json"
        self.res = self.rx.open_url(url)
        res_str = _unicode_to_cn(self.res)
        try:
            res_dict = json.loads(res_str)
        except json.decoder.JSONDecodeError:
            logger.error(f"爬取pms数据失败, 请检查模块 id 是否为: {suite_id}")
            return

        cases = res_dict.get("data").get("cases")
        res_data = deque()
        for run_case_id in cases:
            # 产品库ID
            case_id = cases.get(run_case_id).get("id")
            # 用例库ID
            from_case_id = cases.get(run_case_id).get("fromCaseID")
            case_title = cases.get(run_case_id).get("title")
            # 从用例标题中取出自动化用例id [001]
            at_case_id = re.findall(r"\[(\d{3})\]", case_title)
            # 如果id存在，并且之前没有出现过
            try:
                at_case_id = at_case_id[0]
            except IndexError:
                at_case_id = "   "
            res_data.append(
                {
                    "at_case_id": at_case_id,
                    "case_id": case_id,
                    "from_case_id": from_case_id,
                    "run_case_id": run_case_id,
                    "case_title": case_title,
                }
            )

        return res_data



# -*- coding: utf-8 -*-

# This file was modified by UnionTech Software Technology Co., Ltd. in 2022/08/11

# pylint: disable=C0103,C0209,C0301,E1120,R0913,C0114,C0413,C0413,W0105,C0116,W0702,W0707,R1710,C0411,E0611
from __future__ import absolute_import, division, print_function, unicode_literals
from .config import config
from .utils import doDelay
from .logging import debugLogger as logger

if config.IS_WAYLAND:
    from . import rawinput_wayland
else:
    from pyatspi import Registry as registry
    from pyatspi import (KEY_SYM, KEY_PRESS, KEY_PRESSRELEASE, KEY_RELEASE)

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
from gi.repository import Gdk

"""
Handles raw input using AT-SPI event generation.

Note: Think of keyvals as keysyms, and keynames as keystrings.
"""
__author__ = """
David Malcolm <dmalcolm@redhat.com>,
Zack Cerza <zcerza@redhat.com>
"""


def doTypingDelay():
    doDelay(config.typingDelay)


def checkCoordinates(x, y):
    if x < 0 or y < 0:
        raise ValueError("Attempting to generate a mouse event at negative coordinates: (%s,%s)" % (x, y))


def click(x, y, button=1, check=True):
    """
    Synthesize a mouse button click at (x,y)
    """
    if check:
        checkCoordinates(x, y)
    logger.log("Mouse button %s click at (%s,%s)" % (button, x, y))
    if config.IS_X11:
        registry.generateMouseEvent(x, y, name='b%sc' % button)
    else:
        if button == 1:
            rawinput_wayland.leftClick(x, y)
        elif button == 2:
            rawinput_wayland.middleClick(x, y)
        elif button == 3:
            rawinput_wayland.rightClick(x, y)
        else:
            raise ValueError("参数 button 传入错误！")
    doDelay(config.actionDelay)


def point(x, y, check=True):
    if check:
        checkCoordinates(x, y)
    logger.log("Pointing mouse cursor at (%s,%s)" % (x, y))
    if config.IS_X11:
        registry.generateMouseEvent(x, y, 'abs')
    else:
        rawinput_wayland.moveTo(x, y)
    doDelay(config.actionDelay)


def doubleClick(x, y, button=1, check=True):
    """
    Synthesize a mouse button double-click at (x,y)
    """
    if check:
        checkCoordinates(x, y)
    logger.log("Mouse button %s doubleclick at (%s,%s)" % (button, x, y))
    if config.IS_X11:
        registry.generateMouseEvent(x, y, name='b%sd' % button)
    else:
        rawinput_wayland.doubleClick(x, y)
    doDelay()


def press(x, y, button=1, check=True):
    """
    Synthesize a mouse button press at (x,y)
    """
    if check:
        checkCoordinates(x, y)
    logger.log("Mouse button %s press at (%s,%s)" % (button, x, y))
    if config.IS_X11:
        registry.generateMouseEvent(x, y, name='b%sp' % button)
    else:
        rawinput_wayland.mouseDown(x, y)
    doDelay()


def release(x, y, button=1, check=True):
    """
    Synthesize a mouse button release at (x,y)
    """
    if check:
        checkCoordinates(x, y)
    logger.log("Mouse button %s release at (%s,%s)" % (button, x, y))
    if config.IS_X11:
        registry.generateMouseEvent(x, y, name='b%sr' % button)
    else:
        rawinput_wayland.mouseUp()
    doDelay()


def absoluteMotion(x, y, mouseDelay=None, check=True):
    """
    Synthesize mouse absolute motion to (x,y)
    """
    if check:
        checkCoordinates(x, y)
    logger.log("Mouse absolute motion to (%s,%s)" % (x, y))
    registry.generateMouseEvent(x, y, name='abs')
    if mouseDelay:
        doDelay(mouseDelay)
    else:
        doDelay()


def absoluteMotionWithTrajectory(source_x, source_y, dest_x, dest_y, mouseDelay=None, check=True):
    """
    Synthetize mouse absolute motion with trajectory. The 'trajectory' means that the whole motion
    is divided into several atomic movements which are synthetized separately.
    """
    if check:
        checkCoordinates(source_x, source_y)
        checkCoordinates(dest_x, dest_y)
    logger.log("Mouse absolute motion with trajectory to (%s,%s)" % (dest_x, dest_y))

    dx = float(dest_x - source_x)
    dy = float(dest_y - source_y)
    max_len = max(abs(dx), abs(dy))
    if max_len == 0:
        # actually, no motion requested
        return
    dx /= max_len
    dy /= max_len
    act_x = float(source_x)
    act_y = float(source_y)

    for _ in range(0, int(max_len)):
        act_x += dx
        act_y += dy
        if mouseDelay:
            doDelay(mouseDelay)
        registry.generateMouseEvent(int(act_x), int(act_y), name='abs')

    if mouseDelay:
        doDelay(mouseDelay)
    else:
        doDelay()


def relativeMotion(x, y, mouseDelay=None):
    """
    Synthetize a relative motion from actual position.
    Note: Does not check if the end coordinates are positive.
    """
    logger.log("Mouse relative motion of (%s,%s)" % (x, y))
    registry.generateMouseEvent(x, y, name='rel')
    if mouseDelay:
        doDelay(mouseDelay)
    else:
        doDelay()


def drag(fromXY, toXY, button=1, check=True):
    """
    Synthesize a mouse press, drag, and release on the screen.
    """
    logger.log("Mouse button %s drag from %s to %s" % (button, fromXY, toXY))

    (x, y) = fromXY
    press(x, y, button, check)

    (x, y) = toXY
    absoluteMotion(x, y, check=check)
    doDelay()

    release(x, y, button, check)
    doDelay()


def dragWithTrajectory(fromXY, toXY, button=1, check=True):
    """
    Synthetize a mouse press, drag (including move events), and release on the screen
    """
    logger.log("Mouse button %s drag with trajectory from %s to %s" % (button, fromXY, toXY))

    (x, y) = fromXY
    press(x, y, button, check)

    (x, y) = toXY
    absoluteMotionWithTrajectory(fromXY[0], fromXY[1], x, y, check=check)
    doDelay()

    release(x, y, button, check)
    doDelay()


def typeText(string):
    """
    Types the specified string, one character at a time.
    Please note, you may have to set a higher typing delay,
    if your machine misses/switches the characters typed.
    Needed sometimes on slow setups/VMs typing non-ASCII utf8 chars.
    """
    for char in string:
        pressKey(char)

keyNameAliases = {
    'enter': 'Return',
    'esc': 'Escape',
    'alt': 'Alt_L',
    'control': 'Control_L',
    'ctrl': 'Control_L',
    'shift': 'Shift_L',
    'del': 'Delete',
    'ins': 'Insert',
    'pageup': 'Page_Up',
    'pagedown': 'Page_Down',
    'win': 'Super_L',
    'meta': 'Super_L',
    'super': 'Super_L',
    ' ': 'space',
    '\t': 'Tab',
    '\n': 'Return'
}


def uniCharToKeySym(uniChar):
    i = ord(uniChar)
    keySym = Gdk.unicode_to_keyval(i)
    return keySym


def keyNameToKeySym(keyName):
    keyName = keyNameAliases.get(keyName.lower(), keyName)
    keySym = Gdk.keyval_from_name(keyName)
    # various error 'codes' returned for non-recognized chars in versions of GTK3.X
    if keySym == 0xffffff or keySym == 0x0 or keySym is None:
        try:
            keySym = uniCharToKeySym(keyName)
        except:  # not even valid utf-8 char
            try:  # Last attempt run at a keyName ('Meta_L', 'Dash' ...)
                keySym = getattr(Gdk, 'KEY_' + keyName)
            except AttributeError:
                raise KeyError(keyName)
    return keySym


def keyNameToKeyCode(keyName):
    """
    Use GDK to get the keycode for a given keystring.

    Note that the keycode returned by this function is often incorrect when
    the requested keystring is obtained by holding down the Shift key.

    Generally you should use uniCharToKeySym() and should only need this
    function for nonprintable keys anyway.
    """
    keymap = Gdk.Keymap.get_for_display(Gdk.Display.get_default())
    entries = keymap.get_entries_for_keyval(
        Gdk.keyval_from_name(keyName))
    try:
        return entries[1][0].keycode
    except TypeError:
        pass


def pressKey(keyName):
    """
    Presses (and releases) the key specified by keyName.
    keyName is the English name of the key as seen on the keyboard. Ex: 'enter'
    Names are looked up in Gdk.KEY_ If they are not found there, they are
    looked up by uniCharToKeySym().
    """
    keySym = keyNameToKeySym(keyName)
    registry.generateKeyboardEvent(keySym, None, KEY_SYM)
    doTypingDelay()


def keyCombo(comboString):
    """
    Generates the appropriate keyboard events to simulate a user pressing the
    specified key combination.

    comboString is the representation of the key combo to be generated.
    e.g. '<Control><Alt>p' or '<Control><Shift>PageUp' or '<Control>q'
    """
    strings = []
    for s in comboString.split('<'):
        if s:
            for S in s.split('>'):
                if S:
                    S = keyNameAliases.get(S.lower(), S)
                    strings.append(S)
    for s in strings:
        if not hasattr(Gdk, s):
            if not hasattr(Gdk, 'KEY_' + s):
                raise ValueError("Cannot find key %s" % s)
    modifiers = strings[:-1]
    finalKey = strings[-1]
    for modifier in modifiers:
        code = keyNameToKeyCode(modifier)
        registry.generateKeyboardEvent(code, None, KEY_PRESS)
    code = keyNameToKeyCode(finalKey)
    registry.generateKeyboardEvent(code, None, KEY_PRESSRELEASE)
    for modifier in modifiers:
        code = keyNameToKeyCode(modifier)
        registry.generateKeyboardEvent(code, None, KEY_RELEASE)
    doDelay()


def holdKey(keyName):
    code = keyNameToKeyCode(keyName)
    registry.generateKeyboardEvent(code, None, KEY_PRESS)
    doDelay()


def releaseKey(keyName):
    code = keyNameToKeyCode(keyName)
    registry.generateKeyboardEvent(code, None, KEY_RELEASE)
    doDelay()



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=wrong-import-position
import os
import sys
import traceback
from argparse import ArgumentParser

os.environ["DISPLAY"] = ":0"

from setting.globalconfig import SystemPath

for i in SystemPath:
    if i.value in sys.path:
        continue
    sys.path.append(i.value)

from setting.globalconfig import GlobalConfig
from src.startapp import StartApp
from src import logger
from src.pms.pms2csv import Pms2Csv
from src.pms.csv2pms import Csv2Pms
from src.rtk._base import SubCmd
from src.rtk._base import Args
from src.rtk.local_runner import LocalRunner
from src.rtk.remote_runner import RemoteRunner
from src.depends.cfonts import say
from src.pms.send2pms import Send2Pms


# pylint: disable=too-many-instance-attributes,broad-except
class Manage:
    """执行器"""

    __author__ = "mikigo<huangmingqiang@uniontech.com>"

    # pylint: disable=too-many-arguments,too-many-locals,too-many-statements
    def __init__(
            self,
            app=None,
            keywords=None,
            tags=None,
            rerun=None,
            record_failed_case=None,
            clean=None,
            report_formats=None,
            max_fail=None,
            log_level=None,
            timeout=None,
            resolution=None,
            debug=None,
            noskip=None,
            ifixed=None,
            send_pms=None,
            task_id=None,
            trigger=None,
            case_file=None,
            branch=None,
            deb_path=None,
            pms_user=None,
            pms_password=None,
            suite_id=None,
            pms_info_file=None,
            top=None,
            lastfailed=None,
            duringfail=None,
            repeat=None,
            project_name=None,
            build_location=None,
            line=None,
            client=None,
            send_code=None,
            build_env=None,
            client_password=None,
            parallel=None,
            autostart=None,
            pyid2csv=None,
            export_csv_file=None,
            pms2csv=None,
            csv2pms=None,
            csv_name=None,
            pms_link_csv=None,
            send2task=None,
    ):
        self.default_app = app
        self.default_keywords = keywords
        self.default_tags = tags
        self.default_rerun = rerun
        self.default_record_failed_case = record_failed_case
        self.default_clean = clean
        self.default_report_formats = report_formats
        self.default_max_fail = max_fail
        self.default_log_level = log_level
        self.default_timeout = timeout
        self.default_resolution = resolution
        self.default_debug = debug
        self.default_noskip = noskip
        self.default_ifixed = ifixed
        self.default_send_pms = send_pms
        self.default_task_id = task_id
        self.default_trigger = trigger
        self.default_case_file = case_file
        self.default_branch = branch
        self.default_deb_path = deb_path
        self.default_pms_user = pms_user
        self.default_pms_password = pms_password
        self.default_suite_id = suite_id
        self.default_pms_info_file = pms_info_file
        self.default_top = top
        self.default_lastfailed = lastfailed
        self.default_duringfail = duringfail
        self.default_repeat = repeat
        self.default_project_name = project_name
        self.default_build_location = build_location
        self.default_line = line
        self.default_client = client
        self.default_send_code = send_code
        self.default_build_env = build_env
        self.default_client_password = client_password
        self.default_parallel = parallel
        self.default_autostart = autostart
        self.default_pyid2csv = pyid2csv
        self.default_export_csv_file = export_csv_file
        self.default_pms2csv = pms2csv
        self.default_csv2pms = csv2pms
        self.default_csv_name = csv_name
        self.default_pms_link_csv = pms_link_csv
        self.default_send2task = send2task

        say(GlobalConfig.PROJECT_NAME)
        version_font = "slick"
        if "(unreleased)" in GlobalConfig.current_tag:
            say(GlobalConfig.current_tag.strip("(unreleased)"), font=version_font, space=False)
            say("unreleased", font=version_font, space=False)
        else:
            say(GlobalConfig.current_tag, font=version_font, space=False)
        say(f"Code: \033[0;32m{GlobalConfig.GITHUB_URL}\033[0m", font="console", space=False)
        say(f"Docs: \033[0;32m{GlobalConfig.DOCS_URL}\033[0m", font="console", space=False)
        say(f"PyPI: \033[0;32m{GlobalConfig.PyPI_URL}\033[0m", font="console", space=False)
        say("=" * 60, font="console", space=False)
        logger(GlobalConfig.LOG_LEVEL)

        self.cmd_args = sys.argv[1:]
        parser = ArgumentParser(epilog=self.__author__)
        subparsers = parser.add_subparsers(help="子命令")
        sub_parser_remote = subparsers.add_parser(SubCmd.remote.value)
        sub_parser_run = subparsers.add_parser(SubCmd.run.value)
        sub_parser_pms = subparsers.add_parser(SubCmd.pmsctl.value)
        sub_parser_csv = subparsers.add_parser(SubCmd.csvctl.value)

        help_tip = (
            f"\033[0;32mmanage.py\033[0m 支持 \033[0;32m{[i.value for i in SubCmd]}\033[0m 命令, "
            "\n您需要传入一个命令,可以使用 \033[0;32m-h\033[0m或\033[0;32m--help\033[0m 查看每个命令参数的详细使用说明,"
            "\n比如: \033[0;32myouqu manage.py run -h\033[0m \n"
        )
        if not self.cmd_args:
            print(help_tip)
            sys.exit(1)
        if self.cmd_args[0] == SubCmd.remote.value:
            remote_kwargs = self.remote_runner(parser, sub_parser_remote)
            RemoteRunner(**remote_kwargs).remote_run()
        elif self.cmd_args[0] == SubCmd.run.value:
            _local_kwargs, _ = self.local_runner(parser, sub_parser_run)
            LocalRunner(**_local_kwargs).local_run()
        elif self.cmd_args[0] == SubCmd.pmsctl.value:
            self.pms_control(parser, sub_parser_pms)
        elif self.cmd_args[0] == SubCmd.csvctl.value:
            self.csv_control(parser, sub_parser_csv)
        elif self.cmd_args[0] == SubCmd.startapp.value:
            start_config_log = f"{SubCmd.startapp.value} 后面直接加工程名称,工程名称以 'autotest_' 开头"
            try:
                if self.cmd_args[1] in ("-h", "--help"):
                    print(start_config_log)
                    sys.exit(0)
                self.start_app(self.cmd_args[1])
            except IndexError:
                logger.error(f"参数异常: {start_config_log}")
        elif self.cmd_args[0] in ["-h", "--help"]:
            print(help_tip)
        else:
            print(f"参数异常 \033[0;31m{self.cmd_args}\033[0m!\n{help_tip}")

    def remote_runner(self, parser, sub_parser_remote):
        """远程执行"""
        sub_parser_remote.add_argument(
            "-c", "--clients", default="",
            help=(
                "远程机器的user@ip:password,多个机器用'/'连接,"
                "如果password不传入,默认取setting/remote.ini中CLIENT_PASSWORD的值,"
                "比如: uos@10.8.13.xx:1 或 uos@10.8.13.xx"
            )
        )
        sub_parser_remote.add_argument(
            "-s", "--send_code", action='store_const', const=True, default=False,
            help="发送代码到测试机（不含report目录）"
        )
        sub_parser_remote.add_argument(
            "-e", "--build_env", action='store_const', const=True, default=False,
            help="搭建测试环境,如果为yes，不管send_code是否为yes都会发送代码到测试机."
        )
        sub_parser_remote.add_argument(
            "-cp", "--client_password", default="", help="测试机密码（全局）"
        )
        sub_parser_remote.add_argument(
            "-y", "--parallel", default="",
            help=(
                "yes:表示所有测试机并行跑，执行相同的测试用例;"
                "no:表示测试机分布式执行，服务端会根据收集到的测试用例自动分配给各个测试机执行。"
            )
        )
        local_kwargs, args = self.local_runner(parser, sub_parser_remote)
        remote_kwargs = {
            Args.clients.value: args.clients or self.default_client,
            Args.send_code.value: args.send_code or self.default_send_code,
            Args.build_env.value: args.build_env or self.default_build_env,
            Args.client_password.value: args.client_password or self.default_client_password,
            Args.parallel.value: args.parallel or self.default_parallel,
        }
        _remote_kwargs = {
            "remote_kwargs": remote_kwargs,
            "local_kwargs": local_kwargs,
        }
        return _remote_kwargs

    def local_runner(self, parser, sub_parser_run):
        """本地执行"""
        sub_parser_run.add_argument(
            "-a", "--app", default="",
            help="应用名称：apps/autotest_deepin_music 或 autotest_deepin_music"
        )
        sub_parser_run.add_argument(
            "-k", "--keywords", default="", help="用例的关键词,支持and/or/not逻辑组合"
        )
        sub_parser_run.add_argument(
            "-t", "--tags", default="", help="用例的标签,支持and/or/not逻辑组合"
        )
        sub_parser_run.add_argument(
            "--rerun", default="", help="失败重跑次数"
        )
        sub_parser_run.add_argument(
            "--record_failed_case", default="", help="失败录屏从第几次失败开始录制视频"
        )
        sub_parser_run.add_argument(
            "--clean", choices=["yes", ""], default="",
            help="清理环境"
        )
        sub_parser_run.add_argument(
            "--report_formats", default="", help="测试报告格式"
        )
        sub_parser_run.add_argument(
            "--max_fail", default="", help="最大失败率"
        )
        sub_parser_run.add_argument(
            "--log_level", default="", help="日志输出级别"
        )
        sub_parser_run.add_argument(
            "--timeout", default="", help="单条用例超时时间"
        )
        sub_parser_run.add_argument(
            "--resolution", default="", help="检查分辨率"
        )
        sub_parser_run.add_argument(
            "--debug", default="", help="调试模式"
        )
        sub_parser_run.add_argument(
            "--noskip", choices=["yes", ""], default="",
            help="csv文件里面标记了skip跳过的用例不生效"
        )
        sub_parser_run.add_argument(
            "--ifixed", choices=["yes", ""], default="",
            help="fixed不生效，仅通过skip跳过用例"
        )
        sub_parser_run.add_argument(
            "--send_pms", choices=["", "async", "finish"], default="",
            help="数据回填"
        )
        sub_parser_run.add_argument(
            "--task_id", default="", help="测试单ID"
        )
        sub_parser_run.add_argument(
            "--trigger", choices=["", "auto", "hand"], default="",
            help="触发者"
        )
        sub_parser_run.add_argument(
            "-f", "--case_file", default="", help="根据文件执行用例"
        )
        sub_parser_run.add_argument(
            "--deb_path", default="", help="需要安装deb包的本地路径"
        )
        sub_parser_run.add_argument(
            "-u", "--pms_user", default="", help="pms 用户名"
        )
        sub_parser_run.add_argument(
            "-p", "--pms_password", default="", help="pms 密码"
        )
        sub_parser_run.add_argument(
            "--suite_id", default="", help="pms 测试套ID"
        )
        sub_parser_run.add_argument(
            "--pms_info_file", default="", help="pms 信息文件"
        )
        sub_parser_run.add_argument(
            "--top", default="", help="过程中记录top命令中的值"
        )
        sub_parser_run.add_argument(
            "--lastfailed", action='store_const', const=True, default=False,
            help="仅执行上次失败用例"
        )
        sub_parser_run.add_argument(
            "--duringfail", action='store_const', const=True, default=False,
            help="测试过程中立即显示报错"
        )
        sub_parser_run.add_argument(
            "--repeat", default="", help="指定用例执行次数"
        )
        sub_parser_run.add_argument(
            "--project_name", default="", help="工程名称（写入json文件）"
        )
        sub_parser_run.add_argument(
            "--build_location", default="", help="构建地区（写入json文件）"
        )
        sub_parser_run.add_argument(
            "--line", default="", help="执行的业务线（写入json文件）"
        )
        sub_parser_run.add_argument(
            "--autostart", default="", help="重启类场景开启letmego执行方案"
        )
        args = parser.parse_args()
        local_kwargs = {
            Args.app_name.value: args.app or self.default_app,
            Args.keywords.value: args.keywords or self.default_keywords,
            Args.tags.value: args.tags or self.default_tags,
            Args.reruns.value: args.rerun or self.default_rerun,
            Args.record_failed_case.value: args.record_failed_case or self.default_record_failed_case,
            Args.clean.value: args.clean or self.default_clean,
            Args.report_formats.value: args.report_formats
                                       or self.default_report_formats,
            Args.max_fail.value: args.max_fail or self.default_max_fail,
            Args.log_level.value: args.log_level or self.default_log_level,
            Args.timeout.value: args.timeout or self.default_timeout,
            Args.debug.value: args.debug or self.default_debug,
            Args.noskip.value: args.noskip or self.default_noskip,
            Args.ifixed.value: args.ifixed or self.default_ifixed,
            Args.send_pms.value: args.send_pms or self.default_send_pms,
            Args.task_id.value: args.task_id or self.default_task_id,
            Args.trigger.value: args.trigger or self.default_trigger,
            Args.resolution.value: args.resolution or self.default_resolution,
            Args.case_file.value: args.case_file or self.default_case_file,
            Args.deb_path.value: args.deb_path or self.default_deb_path,
            Args.pms_user.value: args.pms_user or self.default_pms_user,
            Args.pms_password.value: args.pms_password or self.default_pms_password,
            Args.suite_id.value: args.suite_id or self.default_suite_id,
            Args.pms_info_file.value: args.pms_info_file or self.default_pms_info_file,
            Args.top.value: args.top or self.default_top,
            Args.lastfailed.value: args.lastfailed or self.default_lastfailed,
            Args.duringfail.value: args.duringfail or self.default_duringfail,
            Args.repeat.value: args.repeat or self.default_repeat,
            Args.project_name.value: args.project_name or self.default_project_name,
            Args.build_location.value: args.build_location or self.default_build_location,
            Args.line.value: args.line or self.default_line,
            Args.autostart.value: args.autostart or self.default_autostart,
        }
        if local_kwargs.get(Args.autostart.value) or GlobalConfig.AUTOSTART:
            import letmego
            letmego.conf.setting.PASSWORD = GlobalConfig.PASSWORD
            letmego.register_autostart_service(
                user=GlobalConfig.USERNAME,
                working_directory=GlobalConfig.ROOT_DIR,
                cmd=f"pipenv run python manage.py {' '.join(self.cmd_args)}"
            )
        return local_kwargs, args

    def pms_control(self, parser=None, sub_parser_pms=None):
        """pms相关功能命令行参数"""
        sub_parser_pms.add_argument(
            "-a", "--app", default="",
            help="应用名称：apps/autotest_deepin_music 或 autotest_deepin_music"
        )
        sub_parser_pms.add_argument(
            "-u", "--pms_user", default="", help="pms 用户名"
        )
        sub_parser_pms.add_argument(
            "-p", "--pms_password", default="", help="pms 密码"
        )
        sub_parser_pms.add_argument(
            "-plc", "--pms_link_csv", default="",
            help="pms 和 csv 的映射关系，比如：music:81/album:82，多个配置使用'/'分隔"
        )
        sub_parser_pms.add_argument(
            "-p2c", "--pms2csv", action='store_const', const=True, default=False,
            help="从PMS爬取用例标签到csv文件"
        )
        sub_parser_pms.add_argument(
            "-c2p", "--csv2pms", action='store_const', const=True, default=False,
            help="将csv文件里面的标签同步到PMS"
        )
        sub_parser_pms.add_argument(
            "-c", "--csv_name", default="",
            help="将csv文件里面的标签同步到PMS时csv文件的名称"
        )
        sub_parser_pms.add_argument(
            "--send2task",
            choices=["yes", ""],
            default="", help="回填数据到pms测试单"
        )
        sub_parser_pms.add_argument(
            "--task_id", default="", help="测试单ID"
        )
        sub_parser_pms.add_argument(
            "--trigger", choices=["auto", "hand", ""], default="",
            help="触发者"
        )
        args = parser.parse_args()
        pms_kwargs = {
            Args.app_name.value: args.app or self.default_app,
            Args.pms_user.value: args.pms_user or self.default_pms_user,
            Args.pms_password.value: args.pms_password or self.default_pms_password,
            Args.pms2csv.value: args.pms2csv or self.default_pms2csv,
            Args.csv2pms.value: args.csv2pms or self.default_csv2pms,
            Args.csv_name.value: args.csv_name or self.default_csv_name,
            Args.pms_link_csv.value: args.pms_link_csv or self.default_pms_link_csv,
            Args.send2task.value: args.send2task or self.default_send2task,
            Args.task_id.value: args.task_id or GlobalConfig.TASK_ID,
            Args.trigger.value: args.trigger or GlobalConfig.TRIGGER,
        }
        if pms_kwargs.get(Args.pms2csv.value):
            Pms2Csv(
                app_name=pms_kwargs.get(Args.app_name.value),
                user=pms_kwargs.get(Args.pms_user.value) or GlobalConfig.PMS_USER,
                password=pms_kwargs.get(Args.pms_password.value) or GlobalConfig.PMS_PASSWORD,
                pms_link_csv=pms_kwargs.get(Args.pms_link_csv.value),
            ).write_new_csv()
        elif pms_kwargs.get(Args.csv2pms.value):
            Csv2Pms(
                app_name=pms_kwargs.get(Args.app_name.value),
                user=pms_kwargs.get(Args.pms_user.value) or GlobalConfig.PMS_USER,
                password=pms_kwargs.get(Args.pms_password.value) or GlobalConfig.PMS_PASSWORD,
                csv_name=pms_kwargs.get(Args.csv_name.value),
            ).post_to_pms()
        elif (
                pms_kwargs.get(Args.send2task.value)
                and pms_kwargs.get(Args.task_id.value)
                and pms_kwargs.get(Args.trigger.value) == "hand"
        ):
            Send2Pms().send2pms(
                Send2Pms.case_res_path(pms_kwargs.get(Args.task_id.value)),
                Send2Pms.data_send_result_csv(pms_kwargs.get(Args.trigger.value))
            )
        else:
            raise ValueError

    @staticmethod
    def start_app(startapp=None):
        """新建app工程"""
        if startapp:
            start = StartApp(startapp)
            start.copy_template_to_apps()
            start.rewrite()

    def csv_control(self, parser=None, sub_parser_csv=None):
        """csv相关功能命令参数"""
        sub_parser_csv.add_argument(
            "-a", "--app", default="",
            help="应用名称：apps/autotest_deepin_music 或 autotest_deepin_music"
        )
        sub_parser_csv.add_argument(
            "-k", "--keywords", default="", help="用例的关键词"
        )
        sub_parser_csv.add_argument(
            "-t", "--tags", default="", help="用例的标签"
        )
        sub_parser_csv.add_argument(
            "-p2c", "--pyid2csv", action='store_const', const=True, default=False,
            help="将用例py文件的case id同步到对应的csv文件中"
        )
        sub_parser_csv.add_argument(
            "-ec", "--export_csv_file", default="", help="导出csv文件名称，比如：case_list.csv"
        )
        args = parser.parse_args()
        csv_kwargs = {
            Args.app_name.value: args.app or self.default_app,
            Args.keywords.value: args.keywords or self.default_keywords,
            Args.tags.value: args.tags or self.default_tags,
            Args.pyid2csv.value: args.pyid2csv or self.default_pyid2csv or GlobalConfig.PY_ID_TO_CSV,
            Args.export_csv_file.value: args.export_csv_file or self.default_export_csv_file,
            "collection_only": True
        }
        if csv_kwargs.get(Args.pyid2csv.value) or GlobalConfig.PY_ID_TO_CSV:
            from src.csvctl import CsvControl
            _csv = CsvControl(csv_kwargs.get(Args.app_name.value))
            _csv.delete_mark_in_csv_if_not_exists_py()
            _csv.async_mark_to_csv()
        elif csv_kwargs.get(Args.export_csv_file.value):
            LocalRunner(**csv_kwargs).local_run()
        else:
            logger.error(
                f"需要传递一些有用参数或配置项：{Args.pyid2csv.value} 或 {Args.export_csv_file.value}"
                "，您可以使用 -h 或 --help 查看支持的参数"
            )


if __name__ == "__main__":
    try:
        Manage()
    except Exception as exc:
        traceback.print_exception(*sys.exc_info())



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114,C0115,W0621,C0103
from urllib import request
from urllib import parse
from urllib.request import build_opener
from http import cookiejar


class RequestX:
    def __init__(
        self,
        login_url=None,
        headers=None,
        data=None,
    ):
        self.login_url = login_url
        self.headers = headers
        self.data = parse.urlencode(data).encode("utf-8") if data else None
        self.kwargs = {}
        if self.login_url:
            self.kwargs["url"] = self.login_url
        if self.headers:
            self.kwargs["headers"] = self.headers
        if self.data:
            self.kwargs["data"] = self.data

    @property
    def session(self):
        """
         获取opener对象
        :return:
        """
        cookie_jar = cookiejar.CookieJar()
        cookie = request.HTTPCookieProcessor(cookie_jar)
        opener = build_opener(cookie)
        if self.login_url:
            requests = request.Request(**self.kwargs)
            opener.open(requests)
        return opener

    def open_url(self, url, data=None):
        """
         访问url
        :param url:
        :return:
        """
        response = self.session.open(url, data=data).read().decode()
        return response


if __name__ == "__main__":
    user = ""  # 工号
    password = ""  # 密码
    login_url = "https://pms.uniontech.com/user-login-Lw==.html"
    headers = {"content-type": "application/x-www-form-urlencoded; charset=UTF-8"}
    data = {
        "account": user,
        "password": password,
        "passwordStrength": "1",
        "referer": "/zentao/",
        "verifyRand": "370729017",
        "keepLogin": "0",
    }
    # rx = RequestX(login_url=login_url, headers=headers, data=data)
    # url = "https://pms.uniontech.com/testcase-browse-47.html"
    # print(rx.open_url(url))

    rx = RequestX()
    url = "https://www.baidu.com"
    print(rx.open_url(url))



#!/usr/bin/env python3 # pylint: disable=too-many-lines
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

import os
import re
# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114# pylint: disable=C0114
import shutil
from functools import wraps
from time import sleep

import dbus
import letmego

from public.dde_desktop_public_widget.config import Config
from public.right_menu_public_widget.right_menu_public_widget import (
    RightMenuPublicWidget,
)
# from public.dbus_common_widget import DbusCommonWidget
from setting.globalconfig import GlobalConfig
from src import DbusUtils
from src import Src
from src import logger, log, ShortCut
from src.custom_exception import ApplicationStartError
from src.custom_exception import GetWindowInformation
from src.custom_exception import NoIconOfThisSize
from src.custom_exception import TemplateElementNotFound, ElementNotFound
from src.wayland_wininfo import WaylandWindowINfo


def dfm_warning(func):
    """文管版本匹配装饰器"""

    @wraps(func)
    def wraper(*args, **kwargs):
        logger.warning("文管 6.0+ 开始不支持！")
        func(*args, **kwargs)

    return wraper


# pylint: disable=too-many-ancestors
@log
class _DdeDesktopPublicBaseWidget(Src, RightMenuPublicWidget):
    """
    系统桌面操作层封装
    """

    APP_NAME = "dde-desktop"
    DESC = "/usr/bin/dde-desktop"

    def __init__(self, file_box=True):
        self.file_box = file_box
        kwargs = {}
        kwargs["name"] = self.APP_NAME
        kwargs["description"] = self.DESC
        kwargs["check_start"] = True
        kwargs["config_path"] = Config.UI_INI_PATH
        Src.__init__(self, **kwargs)
        setattr(self.ui, "window_info", self.window_info)

    def window_info(self):
        """
         重写底层 window_info
        :return: 窗口信息
        """
        if GlobalConfig.IS_X11:
            try:
                # Get window_operate ID based on package name
                app_id = self.run_cmd(
                    f"xdotool search --classname --onlyvisible {self.APP_NAME}",
                    interrupt=False,
                    out_debug_flag=False,
                    command_log=False,
                ).split("\n")
                app_id_list = [int(_id) for _id in app_id if _id]  # to int
                app_id_list.sort()
                logger.debug(f"app_id_list: {app_id_list}")
                # Obtain the information of the window_operate according to the ID
                actual_id = app_id_list[-1]
                if len(app_id_list) > 1:
                    # 获取所有id的长度
                    _win_size = [[x, self.__window_size(x)[0]] for x in app_id_list]
                    # 从小到大排序
                    _win_size = sorted(
                        [x for x in _win_size if x[1] > 720], key=lambda x: x[1]
                    )
                    if self.file_box:
                        # 取最小的
                        actual_id = _win_size[0][0]
                    else:
                        # 取最大的
                        actual_id = _win_size[-1][0]
                return self.run_cmd(
                    f"xwininfo -id {actual_id}",
                    interrupt=False,
                    out_debug_flag=False,
                    command_log=False,
                )
            except ElementNotFound as exc:
                raise ApplicationStartError(f"{self.APP_NAME, exc}") from exc
        elif GlobalConfig.IS_WAYLAND:
            # 移动到当前窗口
            proxy_object = dbus.SessionBus().get_object("org.kde.KWin", "/dde")
            # 移动鼠标到目标窗口
            dbus.Interface(proxy_object, "org.kde.KWin").WindowMove()
            sleep(1)
            ShortCut.esc()
            return WaylandWindowINfo().window_info()
        return None

    @classmethod
    def __window_size(cls, window_id):
        """
         获取窗口大小
        :param window_id: 窗口ID
        :return: 窗口大小
        """
        try:
            app_window_info = cls.run_cmd(
                f"xwininfo -id {window_id}",
                interrupt=False,
                out_debug_flag=False,
                command_log=False,
            )
            window_width = re.findall(r"Width.*:\s(\d+)", app_window_info)[0]
            window_height = re.findall(r"Height.*:\s(\d+)", app_window_info)[0]
            logger.debug(f"获取窗口大小 {window_width}*{window_height}")
            return int(window_width), int(window_height)
        except (IndexError, KeyError) as exc:
            raise GetWindowInformation(f"获取窗口大小错误 {exc}") from exc

    @staticmethod
    def get_file_or_dir_axes_by_config(name):
        """
         根据桌面配置文件获取文件或者文件夹坐标
        :param name: 文件名称
        :return: 坐标
        """
        from src.desktop import get_desktop_file_location_by_config
        return get_desktop_file_location_by_config(name)

    def click_file_in_desktop_by_config(self, file_name):
        """
         通过配置文件点击桌面文件
        :param file_name: 文件名称
        :return:
        """
        self.click(*self.get_file_or_dir_axes_by_config(file_name))

    def double_click_file_in_desktop_by_config(self, file_name):
        """
         通过配置文件双击桌面文件
        :param file_name: 文件名称
        :return:
        """
        self.double_click(*self.get_file_or_dir_axes_by_config(file_name))
        sleep(2)

    def right_click_file_in_desktop_by_config(self, file_name):
        """
         通过配置文件右键点击桌面文件
        :param file_name: 文件名称
        :return:
        """
        self.right_click(*self.get_file_or_dir_axes_by_config(file_name))

    @staticmethod
    def desktop_dbus():
        """
         桌面的dbus对象
        :return: 桌面 dbus 对象
        """
        return DbusUtils(
            "com.deepin.daemon.Appearance",
            "/com/deepin/daemon/Appearance",
            "com.deepin.daemon.Appearance",
        )

    @staticmethod
    def display_dbus():
        """
         显示器的dbus对象
        :return:
        """
        return DbusUtils(
            "com.deepin.daemon.Display",
            "/com/deepin/daemon/Display",
            "com.deepin.daemon.Display",
        )

    @classmethod
    def find_system_desktop_image(cls, *elements):
        """
         通过图片查找桌面元素坐标
        :param elements:
        :return:
        """
        element = tuple(map(lambda x: f"{Config.PIC_RES_PATH}/{x}", elements))
        return cls.find_image(*element)

    @dfm_warning
    def double_click_file_in_right_view_by_attr(self, file_name):
        """
         双击打开文管右侧文件
        :param file_name: 文件名称
        :return:
        """
        self.dog.find_element_by_attr(f"$//file_view/{file_name}").point()
        # self.dog.app_element("right_view").child(file_name).point()   # 多个computer_window定位失败的问题
        self.double_click()

    def click_element_in_desktop_by_image(self, *pic):
        """
         点击桌面元素
        :param pic: 图片路径
        :return:
        """
        self.click(*self.find_system_desktop_image(*pic))

    def double_click_element_in_desktop_by_image(self, *pic):
        """
         双击桌面元素
        :param pic: 图片路径
        :return:
        """
        self.double_click(*self.find_system_desktop_image(*pic))
        sleep(2)

    def right_click_element_in_desktop_by_image(self, *pic):
        """
         右键桌面元素
        :param pic: 图片路径
        :return:
        """
        self.right_click(*self.find_system_desktop_image(*pic))

    def move_to_element_in_desktop_by_image(self, *pic):
        """
         移动到桌面元素
        :param pic: 图片路径
        :return:
        """
        self.move_to(*self.find_system_desktop_image(*pic))

    @dfm_warning
    def click_dir_in_desktop_plugs_by_ui(self, dir_name, flag):
        """
         调用文管插件，点击指定文件夹
        :param dir_name:  文件名
        :param flag: import 导入窗口， export 导出窗口
        :return:
        """
        if flag in ("import", "export"):
            _x, _y = self.ui.btn_center(dir_name)
            if flag != "import":
                _y = _y - 38
            self.click(_x, _y)
        else:
            logger.error("flag Error")

    @dfm_warning
    def click_file_in_desktop_plugs_by_attr(self, filename):
        """
         调用文管插件，点击指定文件
        :param filename: 文件名称
        :return:
        """
        self.dog.find_element_by_attr(f"$//file_view/{filename}").point()
        # self.dog.app_element('file_view').child(filename).point()
        self.click()

    @dfm_warning
    def double_click_file_in_desktop_plugs_by_attr(self, filename):
        """
         调用文管插件，双击指定文件
        :param filename: 文件名称
        :return:
        """
        self.dog.find_element_by_attr(f"$//file_view/{filename}").point()
        self.double_click()

    @dfm_warning
    def right_click_file_in_desktop_plugs_by_attr(self, filename):
        """
         调用文管插件，右键点击指定文件
        :param filename: 文件名称
        :return:
        """
        self.dog.find_element_by_attr(f"$//file_view/{filename}").point()
        # self.dog.app_element('file_view').child(filename).point()
        self.right_click()


# pylint: disable=too-many-ancestors,too-many-public-methods
@letmego.mark
class DdeDesktopPublicWidget(_DdeDesktopPublicBaseWidget):
    """
    系统桌面业务层
    包含多媒体应用调用文管的文管小窗口
    """

    # ============== 文件选择框 ===========================
    @dfm_warning
    def click_open_btn_in_desktop_plugs_by_ui(self):
        """
         调用文管插件，点击右下角“打开“
        :return:
        """
        self.click(*self.ui.btn_center("打开"))

    @dfm_warning
    def click_cancel_btn_in_desktop_plugs_save_by_ui(self):
        """
         调用文管插件，点击右下角“取消“
        :return:
        """
        self.click(*self.ui.btn_center("取消保存"))

    @dfm_warning
    def click_cancel_btn_in_desktop_plugs_import_by_ui(self):
        """
         调用文管插件，点击右下角“取消“
        :return:
        """
        self.click(*self.ui.btn_center("取消打开"))

    @dfm_warning
    def click_ok_btn_in_pop_up_window_by_ui(self):
        """
         点击小弹窗“关闭”按钮
        :return:
        """
        self.click(*self.ui.btn_center("关闭弹窗-确定"))

    @dfm_warning
    def click_cancel_btn_in_pop_up_window_by_ui(self):
        """
         点击小弹窗的“取消”按钮
        :return:
        """
        self.click(*self.ui.btn_center("关闭弹窗-取消"))

    @dfm_warning
    def click_formate_box_in_desktop_plugs_by_ui(self, flag="import"):
        """
         调用文管插件，点击右下角“格式框“
        :param flag: import 导入插件框，其他，导出插件框
        :return:
        """
        _x, _y = self.ui.btn_center("格式框")
        if flag != "import":
            _y = _y - 40
        self.click(_x, _y)

    @dfm_warning
    def click_recent_dir_in_desktop_plugs_by_ui(self, flag="import"):
        """
         调用文管插件，点击右下角“最近使用“
        :param flag: import 导入插件框，export，导出插件框
        :return:
        """
        if flag in ("import", "export"):
            _x, _y = self.ui.btn_center("最近使用")
            if flag != "import":
                _x, _y = 0, 0
            self.click(_x, _y)
        else:
            logger.error("flag Error")

    @dfm_warning
    def click_home_dir_in_desktop_plugs_by_ui(self, flag="import"):
        """
         调用文管插件，点击右下角“主目录“
        :param flag: import 导入插件框，其他，导出插件框
        :return:
        """
        self.click_dir_in_desktop_plugs_by_ui("主目录", flag)

    @dfm_warning
    def click_desktop_dir_in_desktop_plugs_by_ui(self, flag="import"):
        """
         调用文管插件，点击右下角“桌面“
        :param flag: import 导入插件框，其他，导出插件框
        :return:
        """
        self.click_dir_in_desktop_plugs_by_ui("桌面", flag)

    @dfm_warning
    def click_videos_dir_in_desktop_plugs_by_ui(self, flag="import"):
        """
         调用文管插件，点击右下角“视频“
        :param flag: import 导入插件框，其他，导出插件框
        :return:
        """
        self.click_dir_in_desktop_plugs_by_ui("视频", flag)

    @dfm_warning
    def click_music_dir_in_desktop_plugs_by_ui(self, flag="import"):
        """
         调用文管插件，点击右下角“音乐“
        :param flag: import 导入插件框，其他，导出插件框
        :return:
        """
        self.click_dir_in_desktop_plugs_by_ui("音乐", flag)

    @dfm_warning
    def click_pictures_dir_in_desktop_plugs_by_ui(self, flag="import"):
        """
         调用文管插件，点击右下角“图片“
        :param flag: import 导入插件框，其他，导出插件框
        :return:
        """
        self.click_dir_in_desktop_plugs_by_ui("图片", flag)

    @dfm_warning
    def click_documents_dir_in_desktop_plugs_by_ui(self, flag="import"):
        """
         调用文管插件，点击右下角“文档“
        :param flag: import 导入插件框，其他，导出插件框
        :return:
        """
        self.click_dir_in_desktop_plugs_by_ui("文档", flag)

    @dfm_warning
    def click_downloads_dir_in_desktop_plugs_by_ui(self, flag="import"):
        """
         调用文管插件，点击右下角“下载“
        :param flag: import 导入插件框，其他，导出插件框
        :return:
        """
        self.click_dir_in_desktop_plugs_by_ui("下载", flag)

    @dfm_warning
    def click_computer_dir_in_desktop_plugs_by_ui(self, flag="import"):
        """
         调用文管插件，点击右下角“计算机“
        :param flag: import 导入插件框，其他，导出插件框
        :return:
        """
        self.click_dir_in_desktop_plugs_by_ui("计算机", flag)

    @dfm_warning
    def click_save_btn_in_desktop_plugs_by_ui(self):
        """
         调用文管插件，点击右下角“保存“（左右布局）
        :return:
        """
        self.click(*self.ui.btn_center("保存"))

    @dfm_warning
    def click_blank_space_in_desktop_plugs_by_ui(self):
        """
         点击文管插件空白处
        :return:
        """
        self.click(*self.ui.btn_center("空白处"), _type="xdotool")

    @dfm_warning
    def click_icon_view_btn_in_desktop_plugs_by_ui(self):
        """
         调用文管插件，点击右下角“图标视图“
        :return:
        """
        self.click(*self.ui.btn_center("图标视图"))

    @dfm_warning
    def click_list_view_btn_in_desktop_plugs_by_ui(self):
        """
         调用文管插件，点击右下角“列表视图“
        :return:
        """
        self.click(*self.ui.btn_center("列表视图"))

    @dfm_warning
    def click_first_file_in_desktop_plugs_by_ui(self):
        """
         调用文管插件，点击“列表视图下第一个文件“
        :return:
        """
        self.click(*self.ui.btn_center("列表视图下第一个文件"))

    @dfm_warning
    def double_click_first_file_in_desktop_plugs_by_ui(self):
        """
         调用文管插件，双击“列表视图下第一个文件“
        :return:
        """
        self.double_click(*self.ui.btn_center("列表视图下第一个文件"))

    @dfm_warning
    def click_second_file_in_desktop_plugs_by_ui(self):
        """
         调用文管插件，点击右下角“列表视图下第二个文件“
        :return:
        """
        self.click(*self.ui.btn_center("列表视图下第二个文件"))

    @dfm_warning
    def double_click_second_file_in_desktop_plugs_by_ui(self):
        """
         调用文管插件，双击右下角“列表视图下第二个文件“
        :return:
        """
        self.click(*self.ui.btn_center("列表视图下第二个文件"))

    @dfm_warning
    def double_click_third_file_in_desktop_plugs_by_ui(self):
        """
         调用文管插件，双击“列表视图下第三个文件“
        :return:
        """
        self.double_click(*self.ui.btn_center("列表视图下第三个文件"))

    @dfm_warning
    def click_third_file_in_desktop_plugs_by_ui(self):
        """
         调用文管插件，点击右下角“列表视图下第三个文件“
        :return:
        """
        self.click(*self.ui.btn_center("列表视图下第三个文件"))

    # ============================= 系统桌面 ====================================

    def right_click_new_folder_in_desktop_by_image(self):
        """
         右键点击新建文件夹
        :return:
        """
        self.right_click(*self.find_system_desktop_image("folder"))

    def click_new_folder_in_desktop_by_image(self):
        """
         点击新建文件夹
        :return:
        """
        self.click(*self.find_system_desktop_image("folder"))

    def right_click_center_in_desktop_by_attr(self):
        """
         在桌面中心点点击右键
        :return:
        """
        try:
            self.dog.element_click("screen_canvas_view", button=3)
        except ElementNotFound:
            self.right_click(960, 540)

    def click_center_in_desktop_by_attr(self):
        """
         在桌面中心点点击左键
        :return:
        """
        try:
            self.dog.element_click("screen_canvas_view")
        except ElementNotFound:
            self.click(960, 540)

    def right_click_left_in_desktop_by_attr(self):
        """
         在桌面的左侧1/8处点击右键
        :return:
        """
        try:
            desktop_x, desktop_y = self.dog.element_center("screen_canvas_view")
        except ElementNotFound:
            desktop_x, desktop_y = 960, 520
        self.right_click(desktop_x / 4, desktop_y / 4)

    def click_left_in_desktop_by_attr(self):
        """
         在桌面的左侧1/8处点击
        :return:
        """
        try:
            desktop_x, desktop_y = self.dog.element_center("screen_canvas_view")
        except ElementNotFound:
            desktop_x, desktop_y = 960, 520
        self.click(desktop_x / 4, desktop_y / 4)

    def click_zero_in_desktop_by_attr(self):
        """
         在桌面的0,0区域点击
        :return:
        """
        self.click(0, 0)

    def move_mouse_to_center_and_drag_to_zero(self):
        """
         从桌面中心拖动鼠标到右上角
        :return:
        """
        # 获取中心坐标
        try:
            start_x, start_y = self.dog.element_center("screen_canvas_view")
        except ElementNotFound:
            start_x, start_y = 960, 520
        self.move_on_and_drag_to((start_x, start_y), (0, 0))

    def move_to_desktop_center(self):
        """
         移动鼠标到桌面中心
        :return:
        """
        try:
            self.move_to(*self.dog.element_center("screen_canvas_view"))
        except ElementNotFound:
            self.move_to(960, 520)

    def locate_left_in_desktop_by_attr(self):
        """
         在桌面左边定位
        :return:
        """
        try:
            desktop_x, desktop_y = self.dog.element_center("screen_canvas_view")
        except ElementNotFound:
            desktop_x, desktop_y = 960, 520
        return desktop_x / 2, desktop_y / 2

    def select_two_file_in_desktop_by_attr(self):
        """
         拖动框选两个文件
        :return:
        """
        start_x, start_y = self.locate_left_in_desktop_by_attr()
        rel_x, rel_y = 150, 335
        self.draw_line(start_x - 30, start_y - 30, rel_x, rel_y)

    def create_two_dir_and_select_by_image(self):
        """
         拖动框选两个文件夹并框选他们
        :return:
        """
        self.new_dir_in_desktop_by_cmd("新建文件夹")
        start_x, start_y = self.get_dir_location()
        self.new_dir_in_desktop_by_cmd("新建文件夹1")
        rel_x, rel_y = 150, 335
        self.draw_line(start_x - 30, start_y - 30, rel_x, rel_y)

    def build_new_folder_in_desktop_by_right_menu(self):
        """
         桌面新建名为“新建文件夹”的目录
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_new_folder_in_right_menu_by_image()

    def build_new_word_in_desktop_by_right_menu(self):
        """
         右键桌面新建一个word文档
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_new_document_in_right_menu_by_image()
        self.click_word_in_right_menu_by_image()

    def build_new_excel_in_desktop_by_right_menu(self):
        """
         右键桌面新建一个excel文档
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_new_document_in_right_menu_by_image()
        self.click_excel_in_right_menu_by_image()

    def build_new_ppt_in_desktop_by_right_menu(self):
        """
         右键桌面新建一个ppt文档
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_new_document_in_right_menu_by_image()
        self.click_ppt_in_right_menu_by_image()

    def build_new_txt_in_desktop_by_right_menu(self):
        """
         右键桌面新建一个txt文档
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_new_document_in_right_menu_by_image()
        self.click_txt_in_right_menu_by_image()

    def build_hidden_txt_in_desktop_by_cmd(self):
        """
         命令行创建一个隐藏的文本文档 <.hidden.txt>
        :return:
        """
        self.new_file_in_desktop_by_cmd(".hidden.txt")

    def sort_by_name_in_desktop_by_right_menu(self):
        """
         右键点击以“名称”排序
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_sort_method_in_right_menu_by_image()
        self.click_name_in_right_menu_by_image()

    def sort_by_time_in_desktop_by_right_menu(self):
        """
         右键点击以“修改时间”排序
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_sort_method_in_right_menu_by_image()
        self.click_change_time_in_right_menu_by_image()

    def sort_by_size_in_desktop_by_right_menu(self):
        """
         右键点击以“大小”排序
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_sort_method_in_right_menu_by_image()
        self.click_size_in_right_menu_by_image()

    def sort_by_type_in_desktop_by_right_menu(self):
        """
         右键点击以“类型”排序
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_sort_method_in_right_menu_by_image()
        self.click_type_in_right_menu_by_image()

    # ======图标大小========
    def icon_minimum_in_desktop_by_right_menu(self):
        """
         "右键点击选择图标“极小”
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_icon_size_in_right_menu_by_image()
        self.click_minist_in_right_menu_by_image()

    def icon_small_in_desktop_by_right_menu(self):
        """
         右键点击选择图标“小”
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_icon_size_in_right_menu_by_image()
        self.click_mini_in_right_menu_by_image()

    def icon_middle_in_desktop_by_right_menu(self):
        """
         右键点击选择图标“中”
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_icon_size_in_right_menu_by_image()
        self.click_middle_in_right_menu_by_image()

    def icon_big_in_desktop_by_right_menu(self):
        """
         右键点击选择图标“大”
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_icon_size_in_right_menu_by_image()
        self.click_big_in_right_menu_by_image()

    def icon_maximum_in_desktop_by_right_menu(self):
        """
         右键点击选择图标“极大”
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_icon_size_in_right_menu_by_image()
        self.click_bigist_in_right_menu_by_image()

    def select_icon_size_in_desktop_by_right_menu(self, size):
        """
         右键菜单设置桌面图标大小
        :param size:
        :return:
        """
        size_dict = {
            "极小": self.icon_minimum_in_desktop_by_right_menu,
            "小": self.icon_small_in_desktop_by_right_menu,
            "中": self.icon_middle_in_desktop_by_right_menu,
            "大": self.icon_big_in_desktop_by_right_menu,
            "极大": self.icon_maximum_in_desktop_by_right_menu,
        }
        options = size_dict.get(size)
        if options:
            options()
        else:
            raise NoIconOfThisSize(size)

    def select_auto_sort_in_desktop_by_right_menu(self):
        """
         右键点击“自动排列”
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_auto_sort_in_right_menu_by_image()

    def select_paste_in_desktop_by_right_menu(self):
        """
         桌面右键点击“粘贴”
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_paste_in_right_menu_by_image()

    def select_all_in_desktop_by_right_menu(self):
        """
         桌面右键点击“所有”
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_select_all_in_right_menu_by_image()

    def select_terminal_in_desktop_by_right_menu(self):
        """
         桌面右键点击“在终端中打开”
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_terminal_in_right_menu_by_image()

    def select_setting_display_in_desktop_by_right_menu(self):
        """
         桌面右键点击“显示设置”
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_display_setting_in_right_menu_by_image()

    def select_wallpaper_in_desktop_by_right_menu(self):
        """
         桌面右键点击“壁纸与屏保”
        :return:
        """
        self.right_click_left_in_desktop_by_attr()
        self.click_wallpaper_in_right_menu_by_image()

    def click_file_name_edit_in_desktop_plug_by_ui(self):
        """
         点击文件名修改框
        :return:
        """
        self.click(*self.ui.btn_center("文件名称编辑框"))

    def double_click_computer_in_desktop_by_image(self):
        """
         双击桌面<计算机>图标
        :return:
        """
        self.double_click_element_in_desktop_by_image("deepin_computer")

    def click_computer_in_desktop_by_image(self):
        """
         点击桌面<计算机>图标
        :return:
        """
        self.click_element_in_desktop_by_image("deepin_computer")

    def right_click_computer_in_desktop_by_image(self):
        """
         右键点击计算机桌面图标
        :return:
        """
        try:
            self.right_click_element_in_desktop_by_image("deepin_computer")
        except TemplateElementNotFound:
            self.icon_small_in_desktop_by_right_menu()
            self.right_click_element_in_desktop_by_image("deepin_computer")

    def click_open_computer_in_desktop_by_image(self):
        """
         右键计算机点击打开
        :return:
        """
        self.right_click_computer_in_desktop_by_image()
        self.select_menu(1)

    def click_create_link_computer_in_desktop_by_image(self):
        """
         右键计算机点击创建链接
        :return:
        """
        self.right_click_computer_in_desktop_by_image()
        self.select_menu(2)

    def click_attribute_computer_in_desktop_by_image(self):
        """
         右键计算机点击属性
        :return:
        """
        self.right_click_computer_in_desktop_by_image()
        self.reverse_select_menu(1)

    def drag_computer_down_in_desktop_by_image(self):
        """
         拖动计算机图标
        :return:
        """
        locate = self.find_system_desktop_image("deepin_computer")
        self.move_on_and_drag_to(locate, (0, 340))

    def drag_computer_up_in_desktop_by_image(self):
        """
         拖动计算机图标
        :return:
        """
        locate = self.find_system_desktop_image("deepin_computer")
        self.move_on_and_drag_to(locate, (0, -340))

    def close_computer_attribute_in_desktop_by_image(self):
        """
         关闭计算机属性小窗口
        :return:
        """
        self.click(957, 254)
        self.esc()

    def double_click_trash_in_desktop_by_image(self):
        """
         双击桌面<回收站>图标
        :return:
        """
        try:
            self.double_click_element_in_desktop_by_image("recycle", "recycle_file")
        # 如果找不到，尝试恢复大小后再找
        except TemplateElementNotFound:
            self.icon_small_in_desktop_by_right_menu()
            self.double_click_element_in_desktop_by_image("recycle", "recycle_file")

    def close_trash_attribute_in_desktop_by_image(self):
        """
         关闭回收站属性小窗口
        :return:
        """
        _x, _y = self.find_system_desktop_image("recycle_attribute")
        self.click(_x + 134, _y - 122)

    @dfm_warning
    def click_empty_btn_in_dialog_by_attr(self):
        """
         点击确认窗口中的“清空”
        :return:
        """
        self.dog.app_element("清空").click()

    def right_click_trash_in_desktop_by_image(self):
        """
         右键点击回收站
        :return:
        """
        self.right_click(*self.gettrash_location_by_image())

    def click_trash_in_desktop_by_image(self):
        """
         单机点击回收站
        :return:
        """
        self.click(*self.gettrash_location_by_image())

    def drag_txt_to_trash_on_desktop_by_image(self):
        """
         拖拽txt文件到回收站
        :return:
        """
        self.move_to_txt_on_desktop_by_image()
        self.drag_to_trash_on_desktop_by_image()

    def drag_to_trash_on_desktop_by_image(self):
        """
         拖拽文件到回收站
        :return:
        """
        self.drag_to(*self.gettrash_location_by_image())

    def gettrash_location_by_image(self):
        """
         桌面回收站图标的坐标
        :return:
        """
        return self.find_system_desktop_image("recycle", "recycle_file")

    def get_location_on_desktop_computer_by_image(self):
        """
         桌面计算机图标的坐标
        :return:
        """
        return self.find_system_desktop_image("deepin_computer")

    def right_click_deepin_album_icon_in_desktop_by_image(self):
        """
         在桌面右键相册图标
        :return:
        """
        self.right_click(*self.find_system_desktop_image("deepin_album"))

    def double_click_deepin_album_icon_in_desktop_by_image(self):
        """
         在桌面双击相册图标
        :return:
        """
        self.double_click(*self.find_system_desktop_image("deepin_album"))
        sleep(2)

    def click_open_trash_in_desktop_by_image(self):
        """
         右键回收站点击打开
        :return:
        """
        self.right_click_trash_in_desktop_by_image()
        self.select_menu(1)

    def click_create_link_trash_in_desktop_by_image(self):
        """
         右键回收站点击创建链接
        :return:
        """
        self.right_click_trash_in_desktop_by_image()
        self.reverse_select_menu(2)

    def click_attribute_trash_in_desktop_by_image(self):
        """
         右键回收站点击属性
        :return:
        """
        self.right_click_trash_in_desktop_by_image()
        self.reverse_select_menu(1)

    def drag_trash_down_in_desktop_by_image(self):
        """
         拖动回收站图标
        :return:
        """
        locate = self.gettrash_location_by_image()
        self.move_on_and_drag_to(locate, (0, 340))
        logger.info(f"拖动回收站图标从{locate}至(0, 340)")

    @classmethod
    def create_recursion_dir_by_cmd(cls, path="Desktop", num=4):
        """
         递归生成多个文件夹
        :param path: 路径
        :param num: 生成的文件数
        :return:
        """
        if num == 1:
            os.system(f"mkdir -p ~/{path}/test{num + 1}")
        if num >= 2:
            cmd = "mkdir -p ~/Desktop/test1"
            for i in range(2, num + 1):
                cmd = cmd + f"/test{i}"
            os.system(cmd)

    @classmethod
    def create_long_name_dir_in_desktop_by_cmd(cls):
        """
         在桌面生成超常名字文件夹
        :return:
        """
        os.system(f"mkdir -p ~/Desktop/{'超常名称' * 11}/test")

    @classmethod
    def create_long_name_file_in_desktop_by_cmd(cls):
        """
         在桌面生成超常名字文件
        :return:
        """
        os.system(f"touch  ~/Desktop/{'超常名称' * 11}.txt")

    @classmethod
    def new_files_by_cmd(cls, path="Desktop", num=1):
        """
         生成多个个文件
        :param path: 路径
        :param num: 文件数
        :return:
        """
        for i in range(0, num):
            os.system(f"touch ~/{path}/test{i + 1}.txt")

    @classmethod
    def chmod_dir_in_desktop_by_cmd(cls, permission, dir_name):
        """
         修改桌面文件夹权限
        :param permission: 权限值 例如555,777
        :param dir_name: 文件夹名
        :return:
        """
        os.system(f"chmod -R {permission} ~/Desktop/{dir_name}/")

    @classmethod
    def chmod_file_in_desktop_by_cmd(cls, permission, file_name):
        """
         修改桌面文件权限
        :param permission: 权限值 例如555,777
        :param file_name: 文件名
        :return:
        """
        os.system(f"chmod {permission} ~/Desktop/{file_name}")

    def paste_file_in_desktop_by_right_menu(self):
        """
         在桌面粘贴文件
        :return:
        """
        DdeDesktopPublicWidget.winleft_d()
        sleep(1)
        DdeDesktopPublicWidget.right_click(100, 300)
        self.click_paste_in_right_menu_by_image()

    @classmethod
    def new_dir_in_desktop_by_cmd(cls, *names):
        """
         在桌面上新建文件夹
        :param names: 文件夹名列表
        :return:
        """
        if names:
            for name in names:
                os.system(f"mkdir ~/Desktop/{name} | true")

    @classmethod
    def new_file_in_desktop_by_cmd(cls, *names):
        """
         在桌面上新建txt
        :param names: 文件名列表
        :return:
        """
        if names:
            for name in names:
                os.system(f"touch ~/Desktop/{name}")

    def colleagues_create_different_types_of_files(self):
        """
         在桌面上右键新建ppt,execl,doc,txt文件
        :return:
        """
        self.click_zero_in_desktop_by_attr()
        self.right_click()
        self.click_new_document_in_right_menu_by_image()
        self.click_word_in_right_menu_by_image()
        self.click_zero_in_desktop_by_attr()
        self.right_click()
        self.click_new_document_in_right_menu_by_image()
        self.click_ppt_in_right_menu_by_image()
        self.click_zero_in_desktop_by_attr()
        self.right_click()
        self.click_new_document_in_right_menu_by_image()
        self.click_excel_in_right_menu_by_image()
        self.click_zero_in_desktop_by_attr()
        self.right_click()
        self.click_new_document_in_right_menu_by_image()
        self.click_txt_in_right_menu_by_image()

    @staticmethod
    def copy_music_and_rename_to_desktop_by_cmd(rename):
        """
         复制一首音乐到桌面，并重命名
        :param rename:
        :return:
        """
        shutil.copyfile(
            "/usr/share/music/bensound-sunny.mp3",
            f"/home/{Config.USERNAME}/Desktop/{rename}.mp3",
        )

    @staticmethod
    def copy_movie_to_desktop_by_cmd():
        """
         复制一部视频到桌面
        :return:
        """
        shutil.copyfile(
            "/usr/share/dde-introduction/demo.mp4",
            f"/home/{Config.USERNAME}/Desktop/demo.mp4",
        )

    @staticmethod
    def copy_jpg_and_rename_to_desktop_by_cmd(rename):
        """
         复制一张jpg的图片到桌面，并重命名
        :param rename: 新名字
        :return:
        """
        shutil.copyfile(
            "/usr/share/wallpapers/deepin/abc-123.jpg",
            f"/home/{Config.USERNAME}/Desktop/{rename}.jpg",
        )

    @classmethod
    def desktop_wallpapers_de1_custom_picture(cls):
        """
         删除自定义壁纸目录的图片
         地址：.config/deepin/dde-daemon/appearance/custom-wallpapers
        :return:
        """
        os.system("rm -rf  ~/.config/deepin/dde-daemon/appearance/custom-wallpapers")

    @staticmethod
    def write_txt_in_desktop_by_cmd(text="统信成都测试部", name="测试文档"):
        """
         桌面新建txt并写入文本
        :param text: 文件内容
        :param name: 文件名
        :return:
        """
        with open(f"/home/{Config.USERNAME}/Desktop/{name}.txt", "w", encoding="utf-8") as file:
            file.write(text)

    @staticmethod
    def write_txt_in_desktop_folder_by_cmd(text="统信成都测试部", name="测试文档"):
        """
         桌面新建文件夹内新建txt并写入
        :param text: 文件内容
        :param name: 文件名
        :return:
        """
        _path = os.path.expanduser("~/Desktop/新建文件夹")
        if not os.path.exists(_path):
            os.makedirs(_path)
        with open(f"{_path}/{name}.txt", "w+", encoding="utf-8") as file:
            file.write(text)

    def right_click_txt_on_desktop_by_image(self):
        """
         右键桌面txt文件
        :return:
        """
        self.right_click(*self.find_system_desktop_image("txt"))

    @classmethod
    def get_file_from_ftp(cls, file_name, path="Desktop"):
        """
         从ftp下载psd格式图片
        :param file_name:
        :param path:
        :return:
        """
        os.system(
            f"wget -P  /home/{Config.USERNAME}/{path} "
            f"ftp://{Config.FTP_ADDRESS}/uploads/dde-file-manager/"
            f"automated_test_material/desktop/{file_name} > /dev/null 2&>1"
        )

    def click_txt_on_desktop_by_image(self):
        """
         左键桌面txt文件
        :return:
        """
        self.click_element_in_desktop_by_image("txt")

    def double_click_txt_on_desktop_by_image(self):
        """
         双击桌面txt文件
        :return:
        """
        self.double_click_element_in_desktop_by_image("txt")

    def move_to_txt_on_desktop_by_image(self):
        """
         左移动到桌面txt文件
        :return:
        """
        self.move_to_element_in_desktop_by_image("txt")

    def right_click_folder_in_desktop_by_image(self):
        """
         右键点击文件夹
        :return:
        """
        self.right_click_element_in_desktop_by_image("folder")

    def move_to_folder_in_desktop_by_image(self):
        """
         移动到文件夹
        :return:
        """
        self.move_to_element_in_desktop_by_image("folder")

    def click_folder_in_desktop_by_image(self):
        """
         左键点击文件夹
        :return:
        """
        self.click_element_in_desktop_by_image("folder")

    def double_click_folder_in_desktop_by_image(self):
        """
         双击点击文件夹
        :return:
        """
        self.double_click_element_in_desktop_by_image("folder")

    def double_click_dde_file_manager_icon_in_desktop_by_image(self):
        """
         在桌面双击文管图标
        :return:
        """
        self.double_click(*self.get_dde_file_manager_location_by_image())
        sleep(2)

    def right_click_zip_file_in_desktop_by_image(self):
        """
         右键点击桌面zip压缩包文件
        :return:
        """
        self.right_click_element_in_desktop_by_image("zip")

    def click_only_set_wallpaper_to_desktop_by_image(self):
        """
         点击屏保页面<仅设置桌面>
        :return:
        """
        self.click_element_in_desktop_by_image("only_set_wallpaper_to_desktop")

    @dfm_warning
    def click_display_in_dialog_by_attr(self):
        """
         点击 弹窗中的 “显示“
        :return:
        """
        self.dog.find_element_by_attr("$//显示").click()

    def double_click_zip_on_desktop_by_image(self):
        """
         双击打开zip文件
        :return:
        """
        self.double_click_element_in_desktop_by_image("zip")

    @classmethod
    def drag_txt_to_desktop_zip_by_image(cls):
        """
         拖拽txt到zip文件中
        :return:
        """
        folder_position = DdeDesktopPublicWidget.find_system_desktop_image("txt")
        zip_position = DdeDesktopPublicWidget.find_system_desktop_image("zip")
        cls.click(*folder_position)
        sleep(1)
        cls.drag_to(*zip_position)

    def double_click_no_read_pic_in_desktop_by_img(self):
        """
         桌面双击无读写权限的文件
        :return:
        """
        self.double_click_element_in_desktop_by_image("no_read_pic")

    @classmethod
    def send_icon_to_desktop_by_cmd(cls, app):
        """
         发送图标到桌面
        :param app:
        :return:
        """
        os.system(f"cp /usr/share/applications/{app}.desktop ~/Desktop/")

    def focus_desktop(self):
        """
         聚焦桌面
        :return:
        """
        self.ui.focus_windows("dde-desktop")

    def get_dir_location(self):
        """
         返回桌面文件夹坐标
        :return:
        """
        return self.find_system_desktop_image("folder")

    def get_screensaver_location(self):
        """
         返回屏保界面的屏保按钮坐标
        :return:
        """
        return self.find_system_desktop_image("screensaver")

    def get_setscreensaver_location(self):
        """
         返回屏保界面的设置屏保按钮坐标
        :return:
        """
        return self.find_system_desktop_image("set_screensaver")

    def get_execl_location(self):
        """
         返回桌面execl文件坐标
        :return:
        """
        return self.find_system_desktop_image("execl")

    def get_doc_location(self):
        """
         返回桌面doc文件坐标
        :return:
        """
        return self.find_system_desktop_image("doc")

    def get_ppt_location(self):
        """
         返回桌面ppt文件坐标
        :return:
        """
        return self.find_system_desktop_image("ppt")

    def get_txt_location(self):
        """
         返回桌面txt文件坐标
        :return:
        """
        return self.find_system_desktop_image("txt")

    def get_dde_file_manager_location_by_image(self):
        """
         返回桌面文管文件坐标
        :return:
        """
        return self.find_system_desktop_image("dde_file_manager")

    def get_movie_file_location(self):
        """
         返回桌面视频文件栏坐标
        :return:
        """
        return self.find_system_desktop_image("movie_file")

    # def set_default_workspace(self):
    #     """
    #      在桌面恢复默认工作区
    #     此方法不在提供，请在子项目中重写此方法
    #     :return:
    #     """
    #     # 找寻桌面窗口ID
    #     desktop_id = self.ui.get_windows_id("dde-desktop")
    #     # 聚焦桌面
    #     cmd_f = f"xdotool windowactivate {desktop_id[0]}"
    #     # 使用快捷键切换工作区
    #     cmd_k = "xdotool key super+Right"
    #     bus = DbusCommonWidget()
    #     # 获取总数工作区
    #     workspace_count = bus.get_workspace_count()
    #     # 获取当前工作区
    #     current_workspace = bus.get_current_workspace()
    #     sleep(2)
    #     # 执行聚焦桌面
    #     self.run_cmd(cmd_f, interrupt=False, out_debug_flag=False, command_log=False)
    #     for _ in range(0, int(workspace_count - current_workspace + 1)):
    #         sleep(1)
    #         # 执行快捷键切换工作区
    #         self.run_cmd(
    #             cmd_k, interrupt=False, out_debug_flag=False, command_log=False
    #         )

    def double_click_deepin_music_icon_on_desktop_by_image(self):
        """
         桌面双击音乐图标
        :return:
        """
        self.double_click_element_in_desktop_by_image("deepin_music")

    def right_click_deepin_music_on_desktop_by_image(self):
        """
         右键点击音乐图标
        :return:
        """
        self.right_click_element_in_desktop_by_image("deepin_music")

    def double_click_camera_on_desktop_by_image(self):
        """
         桌面双击相机图标
        :return:
        """
        self.double_click_element_in_desktop_by_image("deepin_camera")

    def double_click_deepin_movie_on_desktop_by_image(self):
        """
         桌面双击相机图标
        :return:
        """
        self.double_click_element_in_desktop_by_image("deepin_movie")

    def double_click_deepin_screen_recorder_on_desktop_by_image(self):
        """
         桌面双击截图录屏图标
        :return:
        """
        self.double_click_element_in_desktop_by_image("deepin_screen_recorder")

    def right_click_deepin_screen_recorder_on_desktop_by_image(self):
        """
         桌面右键截图录屏图标
        :return:
        """
        self.right_click_element_in_desktop_by_image("deepin_screen_recorder")

    def click_deepin_screen_recorder_on_desktop_by_image(self):
        """
         桌面单击截图录屏图标
        :return:
        """
        self.click_element_in_desktop_by_image("deepin_screen_recorder")

    def get_screen_name_by_dbus(self):
        """
         获取当前显示器的名字
        :return:
        """
        return self.display_dbus().get_session_properties_value("Primary")

    def setting_default_wallpaper_by_dbus(self):
        """
         设置壁纸为默认壁纸
        :return:
        """
        self.desktop_dbus().session_object_methods().SetMonitorBackground(
            str(self.get_screen_name_by_dbus()),
            (
                "/usr/share/wallpapers/deepin/desktop"
                f".{'bmp' if Config.SYS_ARCH == Config.ArchName.mips else 'jpg'}"
            ),
        )

    @classmethod
    def new_folder_in_desktop_by_cmd(cls, text=None):
        """
         shell在桌面新建文件夹
        :param text: 文件夹名
        :return:
        """
        os.system(f"mkdir /home/{Config.USERNAME}/Desktop/{text if text else '新建文件夹'}")

    @staticmethod
    def delete_computer_and_trash_icon_in_desktop_by_cmd():
        """
         删除桌面的计算机和回收站图标
        :return:
        """
        try:
            for i in ["dde-computer.desktop", "dde-trash.desktop"]:
                shutil.move(
                    f"/home/{Config.USERNAME}/Desktop/{i}",
                    f"/home/{Config.USERNAME}/Pictures/{i}",
                )
        except FileNotFoundError:
            pass

    @classmethod
    def recovery_computer_and_trash_icon_by_cmd(cls):
        """
         恢复桌面的计算机和回收站图标
        :return:
        """
        for i in ["dde-computer.desktop", "dde-trash.desktop"]:
            cls.run_cmd(
                f"cp /usr/share/applications/{i} /home/{Config.USERNAME}/Desktop/",
                interrupt=False,
                out_debug_flag=False,
                command_log=False,
            )

    @classmethod
    def new_txt_in_desktop_by_cmd(cls, text=None):
        """
         shell在桌面新建txt
        :param text: 文件名
        :return:
        """
        os.system(
            f"touch /home/{Config.USERNAME}/Desktop/{text if text else '新建文本'}.txt"
        )

    @classmethod
    def new_doc_in_desktop_by_cmd(cls, text=None, size=1):
        """
         shell在桌面新建doc
        :param text: 文件名
        :param size: 文件大小
        :return:
        """
        os.system(
            f"dd if=/dev/zero of=/home/{Config.USERNAME}/Desktop/"
            f"{text if text else '新建Word文档'}.doc bs={size}M count=1"
        )

    @classmethod
    def clean_trash_by_cmd(cls):
        """
         清空回收站
        :return:
        """
        os.system(f"rm -rf /home/{Config.USERNAME}/.local/share/Trash/*")

    @dfm_warning
    def double_click_file_in_desktop_plugs_right_view_by_attr(self, element):
        """
         双击桌面目录元素
        :param element: 文件名
        :return:
        """
        self.dog.find_element_by_attr(f"$//file_view/{element}").doubleClick()

    @dfm_warning
    def click_file_in_desktop_plugs_right_view_by_attr(self, element):
        """
         单击桌面目录元素
        :param element: 文件名
        :return:
        """
        self.dog.find_element_by_attr(f"$//file_view/{element}").click()

    def click_delete_btn_in_dialog_by_attr(self):
        """点击确认窗口中的“确认删除”"""
        try:
            self.dog.app_element("删 除").click()
        except ElementNotFound:
            self.dog.app_element("删除").click()

    def click_replace_btn_in_dialog_by_attr(self):
        """点击确认窗口中的“替换”"""
        try:
            self.dog.app_element("替 换").click()
        except ElementNotFound:
            self.dog.app_element("替换").click()

    @classmethod
    def build_new_txt_in_desktop_by_cmd(cls, *names):
        """在桌面上新建一个txt"""
        if names:
            for name in names:
                cls.run_cmd(f"touch ~/Desktop/{name}.txt")
        sleep(0.5)

    @classmethod
    def create_zip_file_in_desktop_by_cmd(cls):
        """
        创建压缩文件
        :return:
        """
        for _i in range(0, 3):
            cls.build_new_txt_in_desktop_by_cmd(f"{_i + 1}.zip")
        cls.run_cmd(r"cd ~/Desktop &&zip auto.zip *.txt && rm *.txt")



# -*- coding: utf-8 -*-  # pylint: disable=too-many-lines

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
from functools import wraps
from time import sleep

import letmego

from public.dde_dock_public_widget.config import Config
from setting.globalconfig import GlobalConfig
from src import ApplicationError, TemplateElementNotFound
from src import Src
from src import log
from src import logger
from src.cmdctl import CmdCtl

APPS = {
    "文件管理器": {"package": "dde-file-manager", "dock_btn": "Btn_文件管理器"},
    "音乐": {"package": "deepin-music", "dock_btn": "Btn_音乐"},
    "影院": {"package": "deepin-movie", "dock_btn": "Btn_影院"},
    "相册": {"package": "deepin-album", "dock_btn": "Btn_相册"},
    "画板": {"package": "deepin-draw", "dock_btn": "Btn_画板"},
    "看图": {"package": "deepin-image-viewer", "dock_btn": "Btn_看图"},
    "截图录屏": {
        "package": "deepin-screen-recorder",
        "dock_btn": "Btn_截图录屏",
    },
    "相机": {"package": "deepin-camera", "dock_btn": "Btn_相机"},
    "控制中心": {"package": "dde-control-center", "dock_btn": "Btn_控制中心"},
    "文本编辑器": {"package": "deepin-editor", "dock_btn": "Btn_文本编辑器"},
    "全局搜索": {
        "package": "dde-grand-search-daemon",
        "dock_btn": "Btn_grand-search",
    },
    "浏览器": {"package": "browser", "dock_btn": "Btn_浏览器"},
    "日历": {"package": "dde-calendar", "dock_btn": "Btn_日历"},
    "应用商店": {
        "ui": "deepin-home-appstore-client",
        "package": "deepin-app-store",
        "dock_btn": "Btn_应用商店",
    },
    "日志收集工具": {
        "package": "deepin-log-viewer",
        "dock_btn": "Btn_日志收集工具"
    },
    "帮助手册": {
        "ui": "dman",
        "package": "deepin-manual",
        "dock_btn": "Btn_帮助手册"
    },
    "字体管理器": {
        "package": "deepin-font-manager",
        "dock_btn": "Btn_字体管理器"
    },
    "归档管理器": {
        "package": "deepin-compressor",
        "dock_btn": "Btn_归档管理器"
    },
    "文档查看器": {
        "package": "deepin-reader",
        "dock_btn": "Btn_文档查看器"
    },
    "系统监视器": {
        "package": "deepin-system-monitor",
        "dock_btn": "Btn_系统监视器"
    },
    "终端": {
        "package": "deepin-terminal",
        "dock_btn": "Btn_终端"
    },
    "计算器": {
        "package": "deepin-calculator",
        "dock_btn": "Btn_计算器"
    },
    "启动盘制作工具": {
        "package": "deepin-boot-maker",
        "dock_btn": "Btn_启动盘制作工具"
    },
    "软件包安装器": {
        "package": "deepin-deb-installer",
        "dock_btn": "Btn_软件包安装器"
    },
    "语音记事本": {
        "package": "deepin-voice-note",
        "dock_btn": "Btn_语音记事本"
    },
    "设备管理器": {
        "package": "deepin-devicemanager",
        "dock_btn": "Btn_设备管理器"
    },
    "安全中心": {
        "package": "deepin-defender",
        "dock_btn": "Btn_安全中心"
    },
    "服务与支持": {
        "package": "uos-service-support",
        "dock_btn": "Btn_服务与支持"
    },
    "连连看": {
        "package": "deepin-lianliankan",
        "dock_btn": "Btn_连连看"
    },
    "五子棋": {
        "package": "deepin-gomoku",
        "dock_btn": "Btn_五子棋"
    },
}


# 检查应用是否配置
def check_application_config(func):
    """check_application_config"""

    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            application = args[1]
        except IndexError:
            application = kwargs.get("application")
        # pylint: disable=consider-iterating-dictionary
        if application not in APPS.keys():
            logger.error("应用不存在或未加入配置")
        else:
            func(*args, **kwargs)

    return wrapper


# 检查应用是否开启状态
def check_process_function(func):
    """check_process_function"""

    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            application = args[1]
        except IndexError:
            application = kwargs.get("application")
        # 文管进程监控与其他多媒体区别
        if application == "dde-file-manager":
            status = CmdCtl.monitor_process(APPS[application]["package"])
        else:
            status = CmdCtl.get_process_status(APPS[application]["package"])

        if status:
            func(*args, **kwargs)
        else:
            raise ApplicationError("应用程序未启动,无法进行此操作")

    return wrapper


# 检查应用是否关闭状态
def check_process_close(func):
    """check_process_close"""

    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            application = args[1]
        except IndexError:
            application = kwargs.get("application")
        # 文管进程监控与其他多媒体区别
        if application == "dde-file-manager":
            status = CmdCtl.monitor_process(APPS[application]["package"])
        else:
            status = CmdCtl.get_process_status(APPS[application]["package"])

        if not status:
            func(*args, **kwargs)
        else:
            raise ApplicationError("应用程序已启动，无法进行此操作")

    return wrapper


# pylint: disable=too-many-ancestors
class BaseWidget(Src):
    """BaseWidget"""

    APP_NAME = "dde-dock"
    DESC = "/usr/bin/dde-dock"

    def __init__(self, check_start=True):
        kwargs = {}
        kwargs["name"] = "dde-dock"
        kwargs["description"] = "/usr/bin/dde-dock"
        kwargs["check_start"] = check_start
        kwargs["config_path"] = Config.UI_INI_PATH
        Src.__init__(self, **kwargs)

    @classmethod
    def find_dock_image(cls, *elements, rate=0.9):
        """
         通过图片查找任务栏元素坐标
        :param elements:
        :return:
        """
        element = tuple(map(lambda x: f"{Config.PIC_RES_PATH}/{x}", elements))
        return cls.find_image(*element, rate=rate)

    def send_link_to_dock(self, application):
        """
         发送快捷方式到任务栏
        :param application:
        :return:
        """
        btn = APPS[application]["dock_btn"]
        if not self.dog.app_element("Form_mainwindow").isChild(btn):
            cmd = (
                "dbus-send --session --print-reply "
                "--dest=com.deepin.dde.daemon.Dock /com/deepin/dde/daemon/"
                "Dock com.deepin.dde.daemon.Dock.RequestDock string:/usr/share/applications/"
                f"{APPS[application]['package']}.desktop int32:0"
            )
            self.run_cmd(
                cmd, interrupt=False, timeout=5, out_debug_flag=False, command_log=False
            )
            sleep(1)
        logger.debug(f"发送<{application}>快捷方式到任务栏")

    @check_application_config
    # @check_process_close
    def click_and_focus_application_in_dock_by_attr(self, application: str):
        """
         在dock启动应用
        :param application: 应用名
        :return:
        """
        # 如果任务栏没有文管，则将文管发送到任务栏
        self.click_application_in_dock_by_attr(application)
        sleep(2)
        self.ui.focus_windows(
            APPS[application]["ui"] if APPS[application].get("ui") else APPS[application]["package"]
        )

    @check_application_config
    def close_app(self, application: str):
        """
         关闭应用
        :param application: 应用名
        """
        package = APPS[application]["package"]
        self.kill_process(package)
        # if package == "dde-file-manager":
        #     self.run_cmd(f"echo '{GlobalConfig.PASSWORD}' | "
        #                  "sudo -S kill -9 `pidof dde-file-manager`",
        #                  interrupt=False,
        #                  out_debug_flag=False,
        #                  command_log=False
        #                  )
        #     self.run_cmd(f"echo '{GlobalConfig.PASSWORD}' | "
        #                  "sudo -S kill -9 `pidof dde-dconfig-daemon`",
        #                  interrupt=False,
        #                  out_debug_flag=False,
        #                  command_log=False)
        #     self.run_cmd(
        #         (
        #             f"echo '{GlobalConfig.PASSWORD}' | sudo -S rm "
        #             "~/.config/dsg/configs/org.deepin.dde.file-manager/"
        #             "org.deepin.dde.file-manager.json"
        #         ),
        #         interrupt=False)
        #     # 临时解决文管隐藏最近使用的Bug
        #     self.run_cmd(
        #         "dde-dconfig --set -a org.deepin.dde.file-manager -r "
        #         "org.deepin.dde.file-manager -k dfm.recent.hidden -v false"
        #     )
        #
        # 删除窗口大小配置文件
        self.run_cmd(
            f"rm -rf /home/{GlobalConfig.USERNAME}/.config/deepin/{package}/{package}*",
            command_log=False,
        )
        # 删除书签配置文件
        self.run_cmd(
            f"rm -f /home/{GlobalConfig.USERNAME}/.config/deepin/{package}.json",
            command_log=False,
        )

    @check_application_config
    def click_application_in_dock_by_attr(self, application: str):
        """
         点击 dock 栏应用图标
        :param application: 应用名
        """
        btn = APPS[application]["dock_btn"]
        self.send_link_to_dock(application)
        self.dog.element_click(btn)
        logger.debug(f"在任务栏点击 <{btn}> 按钮")
        sleep(2)

    @check_application_config
    def right_click_application_in_dock_by_attr(self, application: str):
        """
         右键 dock 栏应用图标
        :param application: 应用名
        """
        btn = APPS[application]["dock_btn"]
        self.send_link_to_dock(application)
        self.dog.element_click(btn, button=3)
        logger.debug(f"在任务栏右键 <{btn}> 按钮")
        sleep(2)

    @check_application_config
    def double_click_application_in_dock_by_attr(self, application: str):
        """
         双击 dock 栏应用图标
        :param application: 应用名
        """
        btn = APPS[application]["dock_btn"]
        self.send_link_to_dock(application)
        self.dog.element_double_click(btn)
        logger.debug(f"在任务栏双击 <{btn}> 按钮")
        sleep(2)

    @check_application_config
    @check_process_function
    def right_click_all_windows_in_dock_by_attr(self, application: str):
        """
         任务栏右键“所有窗口“
        :param application: 应用名
        :return:
        """
        self.send_link_to_dock(application)
        self.right_click_application_in_dock_by_attr(application)
        self.reverse_select_menu(4)
        sleep(0.5)

    @check_application_config
    def right_click_move_in_dock_by_attr(self, application: str):
        """
         任务栏右键“移除驻留”
        :param application: 应用名
        :return:
        """
        self.send_link_to_dock(application)
        package = APPS[application]["package"]
        if application == "dde-file-manager":
            status = CmdCtl.monitor_process(package)
        else:
            status = CmdCtl.get_process_status(package)
        self.right_click_application_in_dock_by_attr(application)
        self.reverse_select_menu(3 if status else 1)
        sleep(0.5)

    @check_application_config
    @check_process_function
    def right_forced_close_in_dock_by_attr(self, application: str):
        """
         任务栏右键“强制退出”
        :param application: 应用名
        :return:
        """
        self.send_link_to_dock(application)
        self.right_click_application_in_dock_by_attr(application)
        self.reverse_select_menu(2)
        sleep(0.5)

    @check_application_config
    @check_process_function
    def right_click_close_all_in_dock_by_attr(self, application: str):
        """
         任务栏右键“关闭所有”
        :param application: 应用名
        :return:
        """
        self.send_link_to_dock(application)
        self.right_click_application_in_dock_by_attr(application)
        self.reverse_select_menu(1)
        sleep(0.5)

    @check_application_config
    @check_process_close
    def right_click_open_in_dock_by_attr(self, application: str):
        """
         任务栏右键“打开”
        :param application: 应用名
        :return:
        """
        self.send_link_to_dock(application)
        self.right_click_application_in_dock_by_attr(application)
        self.reverse_select_menu(2)
        sleep(0.5)

    @check_application_config
    @check_process_function
    def right_click_open_new_windows_in_dock_by_attr(self, application):
        """
         任务栏右键“新建窗口“
        :param application: 应用名
        :return:
        """
        self.send_link_to_dock(application)
        self.right_click_application_in_dock_by_attr(application)
        self.select_menu(2)
        logger.info("任务栏右键“新建窗口“")
        sleep(0.5)

    def right_click_element_in_dock_by_attr(self, btn):
        """在dock上右键某个元素"""
        self.dog.element_click(btn, button=3)


@letmego.mark
@log
# pylint: disable=too-many-public-methods
class DdeDockPublicWidget(BaseWidget):
    """文管对应的dock栏操作"""

    # =============================dock栏 看图操作=======================================
    def open_image_viewer_in_dock_by_attr(self):
        """
         点击任务栏图标启动看图
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("看图")

    def send_image_viewer_to_dock_by_cmd(self):
        """
         发送看图到任务栏
        :return:
        """
        self.send_link_to_dock("看图")

    def click_image_viewer_btn_in_dock_by_attr(self):
        """
         点击任务栏图标启动看图
        :return:
        """
        self.click_application_in_dock_by_attr("看图")

    def right_click_image_viewer_btn_in_dock_by_attr(self):
        """
         右键点击任务栏图标启动看图
        :return:
        """
        self.right_click_application_in_dock_by_attr("看图")

    def close_image_viewer_by_cmd(self):
        """
         关闭看图并清理数据
        :return:
        """
        self.close_app("看图")
        sleep(0.5)

    def get_deepin_image_viewer_location(self):
        """
         返回dock栏的看图图标坐标
        :return:
        """
        # return self.find_system_desktop_image("deepin_image_viewer")
        return self.dog.element_center("Btn_看图")

    # =============================dock栏 文管操作=======================================
    def send_file_manager_to_dock_by_cmd(self):
        """
         发送文件管理器到任务栏
        :return:
        """
        self.send_link_to_dock("文件管理器")

    def open_file_manager_in_dock_by_attr(self):
        """
         点击任务栏图标启动文管
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("文件管理器")

    def click_file_manager_btn_in_dock_by_attr(self):
        """
         点击任务栏图标启动文管
        :return:
        """
        self.click_application_in_dock_by_attr("文件管理器")

    def right_click_open_new_window_in_file_manager_by_attr(self):
        """
         任务栏右键文件管理器“新建窗口“
        :return:
        """
        self.right_click_open_new_windows_in_dock_by_attr("文件管理器")

    def right_click_all_window_in_file_manager_by_attr(self):
        """
         任务栏右键文件管理器“所有窗口“
        :return:
        """
        self.right_click_all_windows_in_dock_by_attr("文件管理器")

    def right_click_file_manager_in_dock_by_attr(self):
        """
         在任务栏右键点击文管
        :return:
        """
        self.right_click_application_in_dock_by_attr("文件管理器")

    def right_click_forced_close_in_file_manager_by_attr(self):
        """
         右键dock栏上强制退出文管
        :return:
        """
        self.right_forced_close_in_dock_by_attr("文件管理器")

    def right_click_close_all_in_file_manager_by_attr(self):
        """
         右键dock栏上文件管理器 点击关闭所有
        :return:
        """
        self.right_click_close_all_in_dock_by_attr("文件管理器")

    def right_click_move_in_file_manager_by_attr(self):
        """
         任务栏右键文件管理器“移除驻留”
        :return:
        """
        self.right_click_move_in_dock_by_attr("文件管理器")

    def close_file_manager_by_cmd(self):
        """
         关闭文管并清理数据
        :return:
        """
        self.close_app("文件管理器")
        sleep(0.5)

    def get_dde_file_manager_location(self):
        """
         返回dock栏文管文件坐标
        :return: 坐标
        """
        # return self.find_dock_image("dde_file_manager")
        return self.dog.element_center("Btn_文件管理器")

    # =============================dock栏 相册操作=======================================
    def double_click_album_in_dock_by_attr(self):
        """
         任务栏双击相册
        :return:
        """
        self.double_click_application_in_dock_by_attr("相册")

    def right_click_album_in_dock_by_attr(self):
        """
         任务栏右键点击相册
        :return:
        """
        self.right_click_application_in_dock_by_attr("相册")

    # =============================dock栏 影院操作=======================================
    def open_movie_in_dock_by_attr(self):
        """
         点击任务栏图标启动影院
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("影院")

    def send_file_deepin_movie_to_dock_by_cmd(self):
        """
         发送影院到任务栏
        :return:
        """
        self.send_link_to_dock("影院")

    def right_click_movie_in_dock_by_attr(self):
        """
         任务栏右键点击影院
        :return:
        """
        self.right_click_application_in_dock_by_attr("影院")

    def click_movie_btn_in_dock_by_attr(self):
        """
         点击任务栏图标启动影院
        :return:
        """
        self.click_application_in_dock_by_attr("影院")

    def close_movie_by_cmd(self):
        """
         关闭影院并清理数据
        :return:
        """
        self.close_app("影院")
        sleep(0.5)

    def get_deepin_movie_app_location(self) -> tuple:
        """
         返回dock栏的影院图标栏坐标
        :return:
        """
        # return self.find_system_desktop_image("deepin_movie")
        return self.dog.element_center("Btn_影院")

    # =============================dock栏 启动器操作=======================================
    def click_launcher_in_dock_by_attr(self):
        """
         任务栏点击启动器
        :return:
        """
        self.esc()
        sleep(1)
        self.dog.element_click("Btn_launcheritem")
        sleep(1)

    # =============================dock栏 右键菜单操作=======================================

    def right_click_launcher_in_dock_by_attr(self):
        """任务栏右键点击启动器,打开任务栏设置菜单
        :return:
        """
        self.dog.element_click("Btn_launcheritem", button=3)
        sleep(1)

    def select_dock_menu(self, number: int):
        """选择任务栏右键菜单中的选项（从上到下）
        :param number: 在菜单中的位置数
        :return:
        """
        self.right_click_launcher_in_dock_by_attr()
        for _ in range(number):
            self.press_key("down")
        sleep(0.5)
        logger.debug(f"选择右键菜单中的选项(从上到下)第{number}项")

    def reverse_select_dock_menu(self, number: int):
        """选择任务栏右键菜单中的选项（从下到上）
        :param number: 在菜单中的位置数
        :return:
        """
        self.right_click_launcher_in_dock_by_attr()
        for _ in range(number):
            self.press_key("up")
        sleep(0.5)
        logger.debug(f"选择右键菜单中的选项(从下到上)第{number}项")

    def select_right_menu_dock_mode(self):
        """任务栏选择右键菜单: 模式
        :return:
        """
        self.select_dock_menu(1)

    def select_right_menu_dock_location(self):
        """任务栏选择右键菜单: 位置
        :return:
        """
        self.select_dock_menu(2)

    def select_right_menu_dock_state(self):
        """任务栏选择右键菜单: 状态
        :return:
        """
        self.reverse_select_dock_menu(2)

    def click_right_menu_dock_set(self):
        """任务栏点击右键菜单: 任务栏设置
        :return:
        """
        self.reverse_select_dock_menu(1)
        self.enter()
        sleep(0.5)

    def click_right_submenu_dock_mode_efficient(self):
        """任务栏点击右键菜单: 模式 -> 高效模式
        :return:
        """
        self.select_right_menu_dock_mode()
        self.enter()
        self.select_submenu(2)
        sleep(0.5)

    def click_right_submenu_dock_mode_fashion(self):
        """任务栏点击右键菜单: 模式 -> 时尚模式
        :return:
        """
        self.select_right_menu_dock_mode()
        self.enter()
        self.select_submenu(1)
        sleep(0.5)

    def click_right_submenu_dock_location_up(self):
        """任务栏点击右键菜单: 位置 -> 上
        :return:
        """
        self.select_right_menu_dock_location()
        self.enter()
        self.select_submenu(1)
        sleep(1)

    def click_right_submenu_dock_location_down(self):
        """任务栏点击右键菜单: 位置 -> 下
        :return:
        """
        self.select_right_menu_dock_location()
        self.enter()
        self.select_submenu(2)
        sleep(1)

    def click_right_submenu_dock_location_left(self):
        """任务栏点击右键菜单: 位置 -> 左
        :return:
        """
        self.select_right_menu_dock_location()
        self.enter()
        self.reverse_select_menu(2)
        sleep(1)

    def click_right_submenu_dock_location_right(self):
        """任务栏点击右键菜单: 位置 -> 右
        :return:
        """
        self.select_right_menu_dock_location()
        self.enter()
        self.reverse_select_menu(1)
        sleep(1)

    def click_right_submenu_dock_state_show(self):
        """任务栏点击右键菜单: 状态 -> 一直显示
        :return:
        """
        self.select_right_menu_dock_state()
        self.enter()
        self.select_submenu(1)
        sleep(1)

    def click_right_submenu_dock_state_hide(self):
        """任务栏点击右键菜单: 状态 -> 一直隐藏
        :return:
        """
        self.select_right_menu_dock_state()
        self.enter()
        self.select_submenu(2)
        sleep(1)

    def click_right_submenu_dock_state_smart_hide(self):
        """任务栏点击右键菜单: 状态 -> 智能隐藏
        :return:
        """
        self.select_right_menu_dock_state()
        self.enter()
        self.reverse_select_menu(1)
        sleep(1)

    # =============================dock栏 显示桌面操作=======================================

    def click_show_desktop_in_dock_by_attr(self):
        """任务栏点击显示桌面
        :return:
        """
        self.dog.element_click("Btn_show-desktop")
        sleep(1)

    def click_show_desktop_area_in_dock_by_attr(self):
        """任务栏点击显示桌面区域
        :return:
        """
        self.dog.element_click("Form_showdesktoparea")
        sleep(1)

    # =============================dock栏 多任务视图操作=======================================

    def click_multitasking_in_dock_by_attr(self):
        """任务栏点击多任务视图
        :return:
        """
        self.dog.element_click("Btn_multitasking")
        sleep(1)

    # =============================dock栏 画板操作=======================================
    def close_draw_by_cmd(self):
        """
         关闭画板并清理数据
        :return:
        """
        self.close_app("画板")
        sleep(0.5)

    def open_draw_in_dock_by_attr(self):
        """
         点击任务栏图标启动画板
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("画板")

    def click_draw_btn_in_dock_by_attr(self):
        """
         点击dock栏的启动画板
        :return:
        """
        self.click_application_in_dock_by_attr("画板")

    def right_click_draw_btn_in_dock_by_attr(self):
        """
         右键点击dock栏的画板录屏
        :return:
        """
        self.right_click_application_in_dock_by_attr("画板")

    # =============================dock栏 回收站操作=======================================
    def open_trash_in_dock_by_attr(self):
        """
         打开dock栏的回收站
        :return:
        """
        self.dog.element_click("Btn_trash")
        sleep(0.5)

    def right_click_trash_in_dock_by_attr(self):
        """
         右键dock栏的回收站
        :return:
        """
        self.dog.element_click("Btn_trash", button=3)
        sleep(0.5)

    def drag_to_trash_in_dock_by_attr(self):
        """
         拖拽到dock栏右边插件区域的回收站
        :return:
        """
        self.drag_to(*self.dog.element_center("Btn_trash"))
        sleep(1)

    def clean_trash_in_dock_by_attr(self):
        """
         dock栏清空回收站
        :return:
        """
        self.right_click_trash_in_dock_by_attr()
        self.select_menu(2)
        sleep(0.5)

    # =============================dock栏 其他操作=======================================
    def click_mount_item_btn_in_dock_by_attr(self):
        """
         在dock栏点击挂载设备按钮
        :return:
        """
        self.dog.element_click("Btn_mount-item-key")

    def right_click_mount_item_btn_in_dock_by_attr(self):
        """
         在dock栏右键点击挂载设备按钮
        :return:
        """
        self.dog.element_click("Btn_mount-item-key", button=3)

    def click_keyboard_btn_in_dock_by_attr(self):
        """
         点击托盘区域软键盘
        :return:
        """
        self.dog.element_click("Btn_onboard")

    def click_disk_in_dock_by_attr(self):
        """
         点击任务栏磁盘图标
        :return:
        """
        self.dog.find_element_by_attr(
            "$//Btn_mainpanelcontrol/Form_trayarea//Form_normalcontainer/Form_mount-item-key"
        ).click()
        sleep(2)  # 响应比较慢

    def click_mount_ftp_icon_in_desktop_by_image(self):
        """
         点击右下角挂载ftp的图标
        :return:
        """
        try:
            _x, _y = self.find_dock_image("mount_ftp_icon_amd")
        except TemplateElementNotFound:
            _x, _y = self.find_dock_image("mount_ftp_icon_arm")
        self.click(_x, _y)

    def click_umount_icon_in_desktop_by_image(self):
        """
         点击右下角挂载ftp的图标
        :return:
        """
        try:
            _x, _y = self.find_dock_image("mount_ftp_icon_amd")
        except TemplateElementNotFound:
            _x, _y = self.find_dock_image("mount_ftp_icon_arm")
        self.click(_x + 243, _y - 3)

    def right_click_disk_usb_in_dock_by_image(self):
        """
         点击磁盘菜单中的U盘
        :return:
        """
        self.right_click(self.find_dock_image("disk_usb", rate=0.8))

    def point_dock_main_window_by_attr(self):
        """
         鼠标移动到任务栏中心坐标
        :return:
        """
        self.move_to(*self.dog.element_center("Form_mainwindow"))

    def click_notifications_in_dock_by_attr(self):
        """
         点击任务栏最右侧通知中心按钮
        :return:
        """
        self.dog.element_click("Btn_notifications")

    # =============================文本编辑器=============================
    def open_editor_in_dock_by_attr(self):
        """
         dock栏点击启动文本编辑器
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("文本编辑器")

    def close_editor_by_cmd(self):
        """
         关闭文本编辑器并清理数据
        :return:
        """
        self.close_app("文本编辑器")

    # =============================dock栏 音乐操作=============================
    def open_music_in_dock_by_attr(self):
        """
         点击任务栏图标启动音乐
        :return:
        """
        if self.get_process_status("deepin-music"):
            self.kill_process("deepin-music")
            sleep(2)
        self.click_and_focus_application_in_dock_by_attr("音乐")

    def right_click_music_btn_in_dock_by_attr(self):
        """
         右键点击音乐
        :return:
        """
        self.right_click_application_in_dock_by_attr("音乐")

    def right_click_music_btn_in_dock_trayarea_by_attr(self):
        """
         右键点击托盘区域音乐图标
        :return:
        """
        self.dog.app_element("Btn_mainpanelcontrol").child(
            "Form_trayarea").child("Form_normalcontainer").child(
            "Btn_deepin-music").click(3)

    def click_music_btn_in_dock_trayarea_by_attr(self):
        """
         点击托盘区域音乐图标
        :return:
        """
        self.dog.app_element("Btn_mainpanelcontrol").child(
            "Form_trayarea").child("Form_normalcontainer").child(
            "Btn_deepin-music").click()

    # def get_deepin_music_location(self) -> tuple:
    #     """
    #      返回dock栏的音乐图标坐标
    #     :return:
    #     """
    #     return self.find_system_desktop_image("deepin_music")

    # =============================dock栏 相机操作=============================
    def open_camera_in_dock_by_attr(self):
        """
         点击任务栏图标启动相机
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("相机")

    def click_camera_btn_in_dock_by_attr(self):
        """
         点击任务栏图标启动相机
        :return:
        """
        self.click_application_in_dock_by_attr("相机")

    # =============================dock栏 控制中心操作=============================
    def click_control_center_in_dock_by_attr(self):
        """
         点击dock栏的启动控制中心
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("控制中心")

    # =============================dock栏 应用商城操作=============================
    def click_app_store_in_dock_by_attr(self):
        """
         点击dock栏的启动应用商店
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("应用商店")

    # =============================dock栏 日历操作=============================
    def click_calendar_in_dock_by_attr(self):
        """
         点击dock栏的启动日历
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("日历")

    def close_calendar_in_dock_by_cmd(self):
        """
         关闭日历
        :return:
        """
        self.close_app("日历")

    # =============================dock栏 浏览器操作=============================
    def click_browser_in_dock_by_attr(self):
        """
         点击dock栏的浏览器
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("浏览器")

    @classmethod
    def close_browser_in_dock_by_cmd(cls):
        """
         点击dock栏的浏览器
        :return:
        """
        cls.kill_process("browser")

    # =============================dock栏 截图录屏===============================
    def send_screen_recorder_to_dock_by_cmd(self):
        """
         发送截图录屏到任务栏
        :return:
        """
        self.send_link_to_dock("截图录屏")

    def click_screen_recorder_in_dock_by_attr(self):
        """
         鼠标左键点击dock栏的启动截图录屏
        :return:
        """
        self.click_application_in_dock_by_attr("截图录屏")

    def right_click_screen_recorder_in_dock_by_attr(self):
        """
         鼠标右键点击dock栏的启动截图录屏
        :return:
        """
        self.right_click_application_in_dock_by_attr("截图录屏")

    def right_click_move_in_screen_recorder_by_attr(self):
        """
         任务栏右键截图录屏“移除驻留”
        :return:
        """
        self.right_click_move_in_dock_by_attr("截图录屏")

    def click_stop_recorder_in_dock_by_attr(self):
        """
         点击任务栏停止录制按钮
        :return:
        """
        _x, _y, _w, _h = self.dog.find_element_by_attr(
            "$//Form_pluginarea/Btn_deepin-screen-recorder-plugin"
        ).extents
        # x, y, w, h = self.dog.app_element(
        # "Form_pluginarea").children("Btn_deepin-screen-recorder-plugin").extents
        self.click(_x + _w / 4, _y + _h / 2)

    # =============================dock栏 相册=======================================
    def open_album_in_dock_by_attr(self):
        """
         dock栏启动相册
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("相册")

    def click_album_btn_in_dock_by_attr(self):
        """
         点击dock栏的启动相册
        :return:
        """
        self.click_application_in_dock_by_attr("相册")

    # =============================dock栏 全局搜索操作=======================================
    def right_click_grand_search_in_dock_by_attr(self):
        """
         右键点击dock栏的全局搜索图标
        :return:
        """
        self.dog.element_click("Btn_grand-search", button=3)
        sleep(0.5)

    def click_grand_search_in_dock_by_attr(self):
        """
         左键点击dock栏的全局搜索图标
        :return:
        """
        self.dog.element_click("Btn_grand-search")
        sleep(0.5)

    def click_set_button_by_attr(self):
        """
         单击全局搜索-搜索设置按钮
        :return:
        """
        self.dog.element_click("Btn_grand-search", button=3)
        self.select_menu(1)
        sleep(0.5)

    def move_to_grand_search_icon_by_attr(self):
        """
         挪动鼠标到dock栏全局搜索图标
        :return:
        """
        _x, _y = self.dog.element_center("Btn_grand-search")
        self.move_to(_x, _y)

    def change_dock_on_desktop_lef(self):
        """
         将任务栏设置在屏幕左侧显示
        :return:
        """
        self.point_dock_main_window_by_attr()
        self.right_click()
        self.select_menu(2)
        sleep(1)
        self.select_menu(2)

    @classmethod
    def change_dock_on_desktop_down(cls):
        """
         将任务栏设置在屏幕下方显示
        :return:
        """
        cls.move_to_and_right_click("0", "600")  # 防止点击到应用图标
        cls.select_menu(2)
        sleep(1)
        cls.select_menu(1)

    @classmethod
    def recovery_dock_default_position_by_cmd(cls):
        """
         恢复dock栏默认下方显示
        :return:
        """
        cls.run_cmd("gsettings set com.deepin.dde.dock position bottom")

    # =============================dock栏 日志收集工具操作=======================================
    def open_log_in_dock_by_attr(self):
        """
         dock栏启动日志收集工具
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("日志收集工具")

    def close_log_by_cmd(self):
        """
         关闭日志收集工具
        :return:
        """
        self.close_app("日志收集工具")
        sleep(0.5)

    # =============================dock栏 帮助手册操作=======================================
    def open_manual_in_dock_by_attr(self):
        """
         dock栏启动帮助手册
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("帮助手册")

    def close_manual_by_cmd(self):
        """
         关闭帮助手册
        :return:
        """
        self.kill_process("dman")
        sleep(0.5)

    # =============================dock栏 字体管理器操作=======================================
    def open_font_manager_in_dock_by_attr(self):
        """
         dock栏启动字体管理器
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("字体管理器")

    def close_font_manager_by_cmd(self):
        """
         关闭字体管理器
        :return:
        """
        self.close_app("字体管理器")
        sleep(0.5)

    # =============================dock栏 归档管理器操作=======================================
    def open_compressor_in_dock_by_attr(self):
        """
         dock栏启动归档管理器
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("归档管理器")

    def close_compressor_by_cmd(self):
        """
         关闭归档管理器
        :return:
        """
        self.close_app("归档管理器")
        sleep(0.5)

    # =============================dock栏 文档查看器操作=======================================
    def open_reader_in_dock_by_attr(self):
        """
         dock栏启动文档查看器
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("文档查看器")

    def close_reader_by_cmd(self):
        """
         关闭文档查看器
        :return:
        """
        self.close_app("文档查看器")
        sleep(0.5)

    # =============================dock栏 系统监视器操作=======================================
    def open_system_monitor_in_dock_by_attr(self):
        """
         dock栏启动系统监视器
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("系统监视器")

    def close_system_monitor_by_cmd(self):
        """
         关闭系统监视器
        :return:
        """
        self.close_app("系统监视器")
        sleep(0.5)

    # =============================dock栏 终端操作=======================================
    def open_terminal_in_dock_by_attr(self):
        """
         dock栏启动终端
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("终端")

    def close_terminal_by_cmd(self):
        """
         关闭终端
        :return:
        """
        self.close_app("终端")
        sleep(0.5)

    # =============================dock栏 计算器操作=======================================
    def open_calculator_in_dock_by_attr(self):
        """
         dock栏启动日志收集工具
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("计算器")

    def close_calculator_by_cmd(self):
        """
         关闭计算器
        :return:
        """
        self.close_app("计算器")
        sleep(0.5)

    # =============================dock栏 启动盘制作工具操作=======================================
    def open_boot_maker_in_dock_by_attr(self):
        """
         dock栏启动日志收集工具
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("启动盘制作工具")

    def close_boot_maker_by_cmd(self):
        """
         关闭日志收集工具
        :return:
        """
        self.close_app("启动盘制作工具")
        sleep(0.5)

    # =============================dock栏 软件包安装器操作=======================================
    def open_deb_installer_in_dock_by_attr(self):
        """
         dock栏启动软件包安装器
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("软件包安装器")

    def close_deb_installer_by_cmd(self):
        """
         关闭软件包安装器
        :return:
        """
        self.close_app("软件包安装器")
        sleep(0.5)

    # =============================dock栏 语音记事本操作=======================================
    def open_voice_note_in_dock_by_attr(self):
        """
         dock栏启动语音记事本
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("语音记事本")

    def close_voice_note_by_cmd(self):
        """
         关闭软件语音记事本
        :return:
        """
        self.close_app("语音记事本")
        sleep(0.5)

    # =============================dock栏 设备管理器操作=======================================
    def open_devicemanager_in_dock_by_attr(self):
        """
         dock栏启动设备管理器
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("设备管理器")

    def close_devicemanager_by_cmd(self):
        """
         关闭软件设备管理器
        :return:
        """
        self.close_app("设备管理器")
        sleep(0.5)

    # =============================dock栏 安全中心操作=======================================
    def open_defender_in_dock_by_attr(self):
        """
         dock栏启动安全中心
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("安全中心")

    def close_defender_by_cmd(self):
        """
         关闭软件安全中心
        :return:
        """
        self.close_app("安全中心")
        sleep(0.5)

    # =============================dock栏 服务与支持操作=======================================
    def open_service_support_in_dock_by_attr(self):
        """
         dock栏启动服务与支持
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("服务与支持")

    def close_service_support_by_cmd(self):
        """
         关闭软件服务与支持
        :return:
        """
        self.close_app("服务与支持")
        sleep(0.5)

    # =============================dock栏 连连看操作=======================================
    def open_lianliankan_in_dock_by_attr(self):
        """
         dock栏启动连连看
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("连连看")

    def close_lianliankan_by_cmd(self):
        """
         关闭软件连连看
        :return:
        """
        self.close_app("连连看")
        sleep(0.5)

    # =============================dock栏 五子棋操作=======================================
    def open_gomoku_in_dock_by_attr(self):
        """
         dock栏启动五子棋
        :return:
        """
        self.click_and_focus_application_in_dock_by_attr("五子棋")

    def close_gomoku_by_cmd(self):
        """
         关闭软件五子棋
        :return:
        """
        self.close_app("五子棋")
        sleep(0.5)



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=E0401,C0413,R0903,W0707,W0611
try:
    import cv2 as cv

    GET_OPENCV_FORM_RPC = False
except ModuleNotFoundError:
    GET_OPENCV_FORM_RPC = True

from image_center import ImageCenter
from image_center.conf import setting as image_setting

from setting.globalconfig import GlobalConfig

image_setting.PORT = 8889
image_setting.SERVER_IP = GlobalConfig.OPENCV_SERVER_HOST

from src import logger


class ImageUtils(ImageCenter):
    """图像识别的工具类"""

    @staticmethod
    def opencv_compare_images_SSIM(
            module_name, imageA_name, imageB_name, ssim=None, app_name=None
    ):
        """
        判断两张图片是否相同
        (临时解决安全中心需要外部依赖的问题，基础框架提供对应的接口;)
        :param module_name:模块
        :param imageA_name:预存图片A
        :param imageB_name:比较图片B
        example:opencv_compare_images_SSIM('system_check', 'system_check_315116', 'pms_315116')
        """
        # pylint: disable=invalid-name,I1101,E1101
        if ssim is None:
            ssim = GlobalConfig.IMAGE_RATE
        picture_path = f"{GlobalConfig.APPS_PATH}/{app_name}/res/picture"
        contrast_path = f"/home/{GlobalConfig.USERNAME}/Pictures/{app_name}"
        file_a = cv.imread(f"{picture_path}/{module_name}/{imageA_name}.png")
        file_b = cv.imread(f"{contrast_path}/{module_name}/{imageB_name}.png")
        file_a = cv.cvtColor(file_a, cv.COLOR_BGR2GRAY)
        file_b = cv.cvtColor(file_b, cv.COLOR_BGR2GRAY)
        result = cv.matchTemplate(file_b, file_a, cv.TM_CCOEFF_NORMED)
        similarity = cv.minMaxLoc(result)[1]
        logger.info("SSIM = " + str(similarity))
        if similarity >= ssim:
            return True
        return False
